#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 异步任务
#  

"""
from celery import Task
from celery import Celery
from celery.schedules import crontab
import logging
import configparser
from oracle.database.modles import Army
from oracle.database import status
from celery.utils.log import get_task_logger

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
console = logging.StreamHandler()
console.setFormatter(formatter)
log.addHandler(console)

logger = get_task_logger('myapp')
logger.setLevel(logging.DEBUG)

app = Celery('tasks')


config = configparser.RawConfigParser()
config.read('/etc/oracle.conf', encoding='utf8')
app.conf.update(
    result_backend='redis://127.0.0.1:6379/0',
    broker_url='redis://127.0.0.1:6379/2',
)


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    # Calls test('hello') every 10 seconds.
    sender.add_periodic_task(10.0, test.s('hello'), name='add every 10')

    # Calls test('world') every 30 seconds
    sender.add_periodic_task(30.0, test.s('world'), expires=10)

    # Executes every Monday morning at 7:30 a.m.
    sender.add_periodic_task(
        crontab(hour=7, minute=30, day_of_week=1),
        test.s('Happy Mondays!'),
    )


class CallbackTask(Task):
    def on_success(self, retval, task_id, args, kwargs):
        print('return: {}'.format(retval))
        log.info('ddddddd')

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        print('return22: {}'.format(task_id))
        log.info('kkkkkkkk')


@app.task(name='tasks.test', debug=True, base=CallbackTask())
def test(arg):
    print(arg)
    log.info('qqqqqqqq')
    return arg

@app.task(name='tasks.test2', debug=True)
def test2(arg):
    print(arg)
    log.info('rrrrrrrrrrrrr')
    return 1

@app.task()
def add(arg1, arg2):
    return arg1+arg2


@app.task(name='tasks.confirmAttribute')
def confirm_attribute(cport, identity, name, value, id=0):
    """
    异步上链
    :param cport:
    :param identity:
    :param name:
    :param value:
    :param id: 数据库中表的id
    :return:
    """
    tf = cport.confirm_attribute(identity, name, value)
    if tf is True:
        Army.select(id=id).update(status == status.EVENT_ETHEREUM_SUBMIT_SUCESS)
    if tf is False:
        Army.select(id=id).update(status == status.EVENT_ETHEREUM_SUBMIT_FAILED)


if __name__ == '__main__':
    result = test.delay(30, 42)
    # 10秒之后运行
    test.apply_async((2, 2), countdown=10)


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 监听器
#  

"""


from oracle.setting import VAR
from pycontractsdk.contracts import Cport
from oracle.database import dbs
from oracle import utils
from oracle.utils import write_blocknum
from pycontractsdk.events import filter_event
import time


class Listener(object):
    def __init__(self):
        self.provider = VAR['ETH_PROVIDER']
        self.contract_address = VAR['CONTRACT_ADDRESS']
        self.abi = VAR['CONTRACT_ABI']
        self.cache_file = VAR['BLOCKNUM_CACHE_FILE']
        self.blocknum_inc = VAR['BLOCKNUM_INC']

        self.army_private_key = VAR['ARMY_PRIVATEKEY']
        self.operator_private_key = VAR['OPERATOR_PRIVATEKEY']
        self.delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
        self.army_blockum_cache_file = VAR['ARMY_BLOCKNUM_CACHE_FILE']


class ArmyListener(Listener):
    def __init__(self):
        super(ArmyListener, self).__init__()

    def army_handle_event(self, entries):
        """
        处理events的业务逻辑
        :param entries:
        :return:
        """
        for entry in entries:
            # 数据存到数据库中
            dbs.save_event(entry)
            block_num = entry['blockNumber']
            write_blocknum(VAR['BLOCKNUM_CACHE_FILE'], block_num)
            args = entry['args']
            identity = args['identity']
            name = str(args['name'], 'utf8')
            event_name = entry['event']
            if event_name == 'DIDAttributeChange':
                # TODO: 解析加密的数据，获得军人的身份数据，去数据库中匹配，
                # TODO: 如果匹配上了就需在数据库中修改这条数据的状态为完成，然后在发送到以太坊中,必须确保一定上链
                # TODO: 如果没有匹配上，就输出log，不做任何处理
                key = self.army_private_key
                if dbs.is_exist_soldier(key):
                    pass
                else:
                    pass
                pass

    def run(self):
        cport = Cport(provider=self.provider, timeout=60, contract_address=self.contract_address, abi=self.abi)
        from_block = utils.get_cache_blocknum(self.army_blockum_cache_file)
        while True:
            block_number = cport.block_number
            try:
                filter_event(
                    provider=self.provider,
                    contract_address=self.contract_address,
                    abi=self.abi,
                    event_name=self.army_handle_event,
                    from_block=from_block,
                    to_block=from_block + self.blocknum_inc
                )
            except Exception as e:
                print(e)
                time.sleep(5)
                continue
            if (from_block + self.blocknum_inc) >= block_number:
                from_block = block_number
            else:
                from_block = from_block + self.blocknum_inc
            write_blocknum(self.army_blockum_cache_file, from_block)
            time.sleep(10)


# -*- coding: utf-8 -*-

"""
存放的是数据库的CRUD操作
"""

from datetime import datetime
import json
import peewee
from oracle.setting import VAR
from oracle.database.modles import (
    Person, PersonDc, Soldiers, Keys, Contract, Events, Army, Organization, db, Account, QueueFlow
)
from oracle.logger import logger
from oracle.exceptions import ContractError
from pycontractsdk.keys import value_to_hex


@db.atomic()
def example_atomic():
    """ 包装器，这个方法是原子的 """
    pass


def example_transaction():
    """ 显示的事务 """
    with db.transaction() as txn:
        Person.create(name='yuyongpeng')
        txn.commit()
        Person.create(name='yuyongpeng2')
        txn.rollback()
    pass


def is_exist_soldier(encrypted_string):
    """
    查询军人的身份信息是否存在于数据库中
    :param encrypted_string:  加密后的身份信息
    :return: 如果查询到了信息就返回True，如果没有查询到就返回False
    """
    count = Soldiers.select().where(encrypted_string == encrypted_string).count()
    if count == 1:
        return True
    else:
        return False


def get_soldier(encrypted_string):
    """ 获得士兵的信息 """
    return Soldiers.get_or_none(encrypted_string=encrypted_string)


def save_soldier(soldier):
    """ 保存士兵的信息 """
    soldier.save()


def query_keys(machine):
    """
    如果数据库中存在这个machine，就直接返回这条数据
    如果数据库中不存在这个machine，就取出一条没有使用的数据，更新状态后返回
    :return: Key对象
    """
    key = Keys.select().where(Keys.machine == machine)
    if len(key) == 1:
        # 如果有数据，就直接使用这条数据
        return key[0]
    if len(key) == 0:
        with db.transaction() as txn:
            # 没有数据，需要得到一条没有使用的key，标识为自己的机器id
            keys = Keys.select().where(Keys.status == 0, ((Keys.machine.is_null()) | (Keys.machine == ''))).limit(1).for_update()
            if len(keys) == 0:
                logger.error("***********************************************************")
                logger.error("* 数据库中没有可使用的key, 程序没法启动，请查看cp_keys表中的数据 *")
                logger.error("***********************************************************")
                exit(2)
            ky = keys[0]
            ky.machine = machine
            ky.status = 1
            ky.updates = datetime.now()
            ky.save()
        return ky
    if len(key) > 1:
        raise ValueError('Table keys where machine=%s count > 1', machine)
    # return key


def query_contract():
    """ 获得一条协约数据 """
    contract = Contract.select().where(Contract.status == 0)
    if len(contract) == 1:
        return contract[0]
    if len(contract) == 0:
        raise ContractError('can not find contract in database')
    if len(contract) > 1:
        raise ContractError('The contract obtained from the database is greater than 1')


def save_event(entry):
    """
    保存event
    :param entry:
    :return:
    """
    args = entry['args']
    event_name = entry['event']
    log_index = entry['logIndex']
    transaction_index = entry['transactionIndex']
    transaction_hash = value_to_hex(entry['transactionHash'])
    address = entry['address']
    block_hash = value_to_hex(entry['blockHash'])
    block_number = entry['blockNumber']
    # 根据table中的唯一索引来查询
    # 先插入数据，如果抛出了唯一索引的异常，就直接更新数据
    print(entry)
    try:
        with db.atomic():
            event = Events.create(
                args=str(args),
                event_name=event_name,
                log_index=log_index,
                transaction_index=transaction_index,
                transaction_hash=transaction_hash,
                address=address,
                block_hash=block_hash,
                block_number=block_number,
            )
        return event
    except peewee.IntegrityError:
        # `username` is a unique column, so this username already exists,
        # making it safe to call .get().
        # query = Events.update(
        #     args=str(args),
        #     log_index=log_index,
        #     address=address,
        #     block_hash=block_hash,
        #     block_number=block_number,
        #     updates=datetime.datetime.now(),
        # ).where(
        #     Events.event_name == event_name,
        #     Events.transaction_index == transaction_index,
        #     Events.transaction_hash == transaction_hash,
        # )
        # query.execute()
        event = Events.get(
            Events.args == str(args),
            Events.event_name == event_name,
            Events.log_index == log_index,
            Events.transaction_hash == transaction_hash,
            Events.address == address,
            Events.block_hash == block_hash,
            Events.block_number == block_number
        )
        event.updates = datetime.now()
        event.save()
        return event
    # 这种实现方式不好 ，没有都会先查询
    # event = Events.get_or_none(
    #     event_name=event_name,
    #     transaction_index=transaction_index,
    #     transaction_hash=transaction_hash,
    # )
    # if event is None:
    #     Events.create(
    #         args=str(args),
    #         event_name=event_name,
    #         log_index=log_index,
    #         transaction_index=transaction_index,
    #         transaction_hash=transaction_hash,
    #         address=address,
    #         block_hash=block_hash,
    #         block_number=block_number,
    #     )
    # else:
    #     event.updates = datetime.datetime.now()
    #     event.save()


def save_army(entry):
    """
    保存军队使用的event
    :param entry:
    :return:
    """
    args = entry['args']
    identity = args['identity']                         # event 参数 identity 的数据
    name = args['name'].strip(b'\x00')                  # event 参数 name 的数据
    value = args['value']                               # event 参数 value 的数据 (did + \n + 签名信息)
    sign_text_bytes = value.split(b'\n')[-1]            # 第一个 签名信息
    did_json_bytes = b''.join(value.split(b'\n')[:-1])  # did 信息 bytes
    did_json_obj = json.loads(did_json_bytes.decode())  # did json对象

    event_name = entry['event']
    log_index = entry['logIndex']
    transaction_index = entry['transactionIndex']
    transaction_hash = value_to_hex(entry['transactionHash'])
    address = entry['address']
    block_hash = value_to_hex(entry['blockHash'])
    block_number = entry['blockNumber']
    # 根据table中的唯一索引来查询
    # 先插入数据，如果抛出了唯一索引的异常，就直接更新数据
    print(entry)
    try:
        with db.atomic():
            event = Army.create(
                args=str(args),
                identity=identity,
                event_name=event_name,
                log_index=log_index,
                transaction_index=transaction_index,
                transaction_hash=transaction_hash,
                address=address,
                block_hash=block_hash,
                block_number=block_number,
                first_sign=sign_text_bytes.decode(),
                did_json=did_json_bytes.decode(),
                created=did_json_obj['created'],
                created2=datetime.fromtimestamp(int(did_json_obj['created'] / 1000)),
                did_id=did_json_obj['id']
            )
        return event
    except peewee.IntegrityError as e:
        print(e)
        event = Army.get(
            # Army.args == str(args),
            Army.event_name == event_name,
            # Army.log_index == log_index,
            Army.transaction_hash == transaction_hash,
            Army.transaction_index == transaction_index,
            # Army.address == address,
            # Army.block_hash == block_hash,
            # Army.block_number == block_number
        )
        event.updates = datetime.now()
        event.save()
        # event.update()
        return event


def save_organization(entry):
    """
    保存 机构 使用的event
    :param entry: event数据
    :return: 返回到数据库的这个实例
    """
    args = entry['args']
    identity = args['identity']                         # event 参数 identity 的数据
    name = args['name'].strip(b'\x00')                  # event 参数 name 的数据
    value = args['value']                               # event 参数 value 的数据 (did + \n + 签名信息)
    first_sign_text_bytes = value.split(b'\n')[-2]      # 第一次的签名信息 （个人用户）
    sencond_sign_text_bytes = value.split(b'\n')[-1]    # 第二次的签名信息 （军队）
    did_json_bytes = b''.join(value.split(b'\n')[:-2])  # did 信息 byte
    did_json_obj = json.loads(did_json_bytes.decode())  # did json对象
    event_name = entry['event']
    log_index = entry['logIndex']
    transaction_index = entry['transactionIndex']
    transaction_hash = value_to_hex(entry['transactionHash'])
    address = entry['address']
    block_hash = value_to_hex(entry['blockHash'])
    block_number = entry['blockNumber']
    # 根据table中的唯一索引来查询
    # 先插入数据，如果抛出了唯一索引的异常，就直接更新数据
    # logger.info(entry)
    try:
        with db.atomic():
            event = Organization.create(
                args=str(args),
                identity=identity,
                event_name=event_name,
                log_index=log_index,
                transaction_index=transaction_index,
                transaction_hash=transaction_hash,
                address=address,
                block_hash=block_hash,
                block_number=block_number,
                did_json=did_json_bytes.decode(),
                first_sign=first_sign_text_bytes.decode(),
                second_sign=sencond_sign_text_bytes.decode(),
                created=did_json_obj['created'],
                did_id=did_json_obj['id'],
                mark=name
            )
        return event
    except peewee.IntegrityError as e:
        logger.error(e)
        event = Organization.get(
            # Organization.args == str(args),
            Organization.event_name == event_name,
            # Organization.log_index == log_index,
            Organization.transaction_hash == transaction_hash,
            Organization.transaction_index == transaction_index,
            # Organization.address == address,
            # Organization.block_hash == block_hash,
            # Organization.block_number == block_number
        )
        event.updates = datetime.now()
        event.save()
        return event


def save_organization_did(entry):
    """
    保存 机构 验证通过的数据
    :param entry: event数据
    :return: 返回到数据库的这个实例
    """
    args = entry['args']
    identity = args['identity']                         # event 参数 identity 的数据
    name = args['name'].strip(b'\x00')                  # event 参数 name 的数据
    value = args['value']                               # event 参数 value 的数据 (did + \n + 签名信息 + \n + 签名信息)
    first_sign_text_bytes = value.split(b'\n')[-2]      # 第一次的签名信息 （个人用户）
    sencond_sign_text_bytes = value.split(b'\n')[-1]    # 第二次的签名信息 （军队）
    did_json_bytes = b''.join(value.split(b'\n')[:-2])  # did 信息 byte
    did_json_obj = json.loads(did_json_bytes.decode())  # did json对象
    event_name = entry['event']
    log_index = entry['logIndex']
    transaction_index = entry['transactionIndex']
    transaction_hash = value_to_hex(entry['transactionHash'])
    address = entry['address']
    block_hash = value_to_hex(entry['blockHash'])
    block_number = entry['blockNumber']

    # 根据table中的唯一索引来查询
    # 先插入数据，如果抛出了唯一索引的异常，就直接更新数据
    print(entry)
    try:
        with db.atomic():
            Organization.create(
                args=str(args),
                identity=identity,
                event_name=event_name,
                log_index=log_index,
                transaction_index=transaction_index,
                transaction_hash=transaction_hash,
                address=address,
                block_hash=block_hash,
                block_number=block_number,
                first_sign=first_sign_text_bytes.decode(),
                second_sign=sencond_sign_text_bytes.decode(),
                did_json=did_json_bytes.decode(),
                created=did_json_obj['created'],
                did_id=did_json_obj['id'],      # 唯一索引
                # did_json_obj['id'].split()[-1] if len(did_json_obj['id'].split()) > 1 else '',
                mark=name

            )
    except peewee.IntegrityError as e:
        logger.error(e)
        query = Organization.update(
            args=str(args),
            event_name=event_name,
            log_index=log_index,
            transaction_index=transaction_index,
            transaction_hash=transaction_hash,
            address=address,
            block_hash=block_hash,
            block_number=block_number,
            updates=datetime.now(),
            first_sign=first_sign_text_bytes.decode(),
            second_sign=sencond_sign_text_bytes.decode(),
            did_json=did_json_bytes.decode(),
            created=did_json_obj['created'],
            did_id=did_json_obj['id'],
            identity=did_json_obj['id'].split()[-1] if len(did_json_obj['id'].split()) > 1 else ''
        ).where(
            Organization.did_id == did_json_obj['id']
        )
        query.execute()


def get_openid(address: str) -> str:
    """
    根据用户的address查询这个用户对应的openid
    :param address:
    :return:
    """
    with db.atomic():
        query = (Person.select(Person, Account).join(Account).where(Person.address == address))
        if query.count < 1:
            return None
        else:
            return query[0].account.weixin_openid



def save_person_dc(name: str, sign: str, identity: str, organization_hash: str = '',
                   organization_address: str = '') -> PersonDc:
    """
    保存数据到 person_dc 中
    :param name:
    :param sign:
    :param identity:
    :return:
    """
    persondc = PersonDc.create(value=name, person_hash=sign,
                    person_address=identity,
                    organization_hash=organization_hash,
                    organization_address=organization_address)
    return persondc

    pass

def update_complited(status: bool):
    """
    上链完成后调用，更新数据库的状态
    :param status: 需要更新的状态值
    :return:
    """
    # TODO: 需要和学文确定具体的表和列和状态值

    pass

def get_queue_flow(entity):
    """
    根据队列中的json串建立对象
    :param entity:
    :return:
    """
    obj = json.loads(entity)
    id = obj.get('queue_flow', 0)
    type = obj.get('type')
    retry = obj.get('retry', 0)
    upload_time = obj.get('upload_time', '')
    receipt = obj.get('receipt', '')
    if isinstance(entity, bytes):
        data = entity.decode()
    creates = datetime.now()
    updates = datetime.now()
    if id == 0:
        with db.atomic():
            queue = QueueFlow.create(
            type=type,
            retry=retry,
            upload_time=datetime.now() if upload_time is ''  else upload_time,
            receipt=receipt,
            data=data,
            creates=creates,
            updates=updates
            )
    else:
        queue = QueueFlow.get_by_id(id)
    return queue


def update_mysql(success, *args, **kwargs):
    """
    更新数据库的状态
    :param success:  True: 成功上链，False: 上链失败
    :param args:
    :param kwargs:
    :return:
    """
    ip = kwargs.pop('ip', VAR['MYSQL_IP'])
    port = kwargs.pop('port', VAR['MYSQL_PORT'])
    user = kwargs.pop('user', VAR['MYSQL_USER'])
    password = kwargs.pop('password', VAR['MYSQL_PASSWORD'])
    charset = VAR['MYSQL_CHARSET']
    schema = kwargs.pop('schema', VAR['MYSQL_SCHEMA'])
    table = kwargs.pop('table')
    column = kwargs.pop('column')
    id_column = kwargs.pop('id_column')
    id = kwargs.pop('id')
    db = peewee.MySQLDatabase(
        host=ip,
        database=schema,
        port=port,
        user=user,
        passwd=password,
        charset=charset
    )
    value = kwargs.pop('success_tag') if success else kwargs.pop('failed_tag')
    sql = "update {}.{} set {}=%s where {}=%s".format(schema, table, column, id_column)
    # sql = "update {}.{} set {}={} where {}={}".format(schema, table, column, value, id_column, id)
    print(sql)
    cursor = db.execute_sql(sql, (value, id))
    db.commit()
    cursor.close()
    db.close()

















#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 加密解密的操作
#  

"""

import rsa
import base64
import shlex, subprocess
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5


def encrypt(msg, pub_key, encode='utf-8'):
    """
    使用公钥对字符串加密
    :param msg: 需要加密的字符串
    :param pub_key: 公钥
    :param encode: 编码
    :return: 加密后的字符串
    """
    pubkey = rsa.PublicKey.load_pkcs1(pub_key.encode(encode))
    crypto = rsa.encrypt(msg.encode(encode), pubkey)
    return crypto


def decrypt(crypto, pri_key, encode='utf-8'):
    """
    使用私钥界面字符串
    :param crypto: 加密的字符串
    :param pri_key: 私钥
    :param encode: 编码
    :return: 加密后的字符串
    """
    privkey = rsa.PrivateKey.load_pkcs1(pri_key.encode(encode))
    msg = rsa.decrypt(crypto, privkey)
    return msg


rsa_private_key = b'-----BEGIN RSA PRIVATE KEY-----\nMIIEowIBAAKCAQEAz4C8rDjl6NAAvdUxLl7p8eA+xxc4Sa20puq2gFPk+8B5W30o+8xAaktLoji9wEsf7WTEllxZcIXw7d4K9b1M34ZDQT1iR4ZreB3GkxEXDeQIV3f/W7C3NL1bF5ePn7DRGZYKTEry8pp/lK/1ESP9Z+giaN29xm+jsmvNAOJlLjndn8nYcmA1sdDzuX8JSlZOiijAwyKUkwrBd+FqlhbAfF9ebcEgQcRISZsBMpcsZ66HQ0nvEnhmLzkfXxLUu2w7ccwLc/W7a4V1SNFEMZifRyiI7IhYYpRFw7egQN2RgzVWW/O6HKzk2Mxaz/6Y+HEehuDfRDn5MjMCkhnfbpRKKwIDAQABAoIBAD7xpkn32IYO0qrDCPAwCnc8ts5d1M1EzA/2EvZKfKOBd06xYMVA6ClWPdMvsx+pqj2w25Fg4iCoALWnLVvt2GvVpoEbxbw8ok0Fez8RaBTmn0EpPWTq7igpaIB3kTG3yTYhtW3/LkM5aooH9icE69GhMQ7udKrFIrr0yHMEOf0TNKBUNlwwHWJrpUuodXxIbERonkNdri1AnsWsnh4ePkKFlgaIdOZ24W4kHjfQAQkz3bPTjRzU5Q0LpkCpuYxk+TeOvR5tNPHXtzTWYek7JbCbnIKr1SiJx+ncwWM7QEnuvbzCU2VeTwb6Rq94WIg3RpfqSKJPr5w5e/df4jmi58ECgYEA74ty4UiChdp27/xvy2sJMilgkcL5Z0A+McPrjM82CqfPvr1+ZMBkTwuQGxFWlNtdE7DLpneVCWmzAexUJ02dFndSJMqsN81335VYfLc94qk5HXwmc9UBK7+qL2Ugyz76KoRGogqkKYw0ZR2kdpcNoXjQfeVgqrumt+jnZChpAl8CgYEA3cHPuDbdqqhcslz4+Zz8B0zWKN8Kj5CgG9ReEmJGpE6w5D7bds5iPcmp5xwFctuumAtroA/+bpGX/2oXc1nXucf/dI3AWoaBQrZ9iL6xfe7+ZPiRYuZjz3ygZ+Vcgp6xixbTT86jaIHwsD1wYtzOiiYOUBt4jDk89xookjpBg7UCgYEAtCriQOdUpHBoPKBVRV5AEGZmp7tJ8oqzPKLrK4E+WE1XR1MnGYhK6dj8jh9AS72iCAlVYuNWSToi1TN3KAiMOjWMpvd5CI+VtaiSYVtBRJWay75w/XYb51fFHNinDbdUWV4b8gym5Ej7r4HYDQoXyncf+VDooAF7p5+ZSg/Ky2kCgYAlVsv+Dab3ZE/vbH4zsX4yUHrC+QNTNxvuc6y/VbLlWaapV/gmIgwisUEde/di5qCYU1v04JyLy5IXXKrELn2Hd6iI3JFl7L35GLc8fjdup/5HzB7W00o9FpP/ynvwNn5YmChOiNG0+CQp2L6CtwFD+7JvRKDgo4ajaNEqBgsf/QKBgE6kbfW6FSpFdOs+2qpIXTEWnTqUC3WD1h1qB8G8J3Tn/o/bSm8C5lHmjmg9vRJtHisaoLyc2ikH7UZwPGU32340s22igkDZml730CKrYtnnBZAG+tIiERtI4Zj432X7qzBYgTgwO7wRAvds+WDCwwV5FMGNCzbPEXos8HmS0812\n-----END RSA PRIVATE KEY-----\n'
rsa_public_key = b'-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAz4C8rDjl6NAAvdUxLl7p8eA+xxc4Sa20puq2gFPk+8B5W30o+8xAaktLoji9wEsf7WTEllxZcIXw7d4K9b1M34ZDQT1iR4ZreB3GkxEXDeQIV3f/W7C3NL1bF5ePn7DRGZYKTEry8pp/lK/1ESP9Z+giaN29xm+jsmvNAOJlLjndn8nYcmA1sdDzuX8JSlZOiijAwyKUkwrBd+FqlhbAfF9ebcEgQcRISZsBMpcsZ66HQ0nvEnhmLzkfXxLUu2w7ccwLc/W7a4V1SNFEMZifRyiI7IhYYpRFw7egQN2RgzVWW/O6HKzk2Mxaz/6Y+HEehuDfRDn5MjMCkhnfbpRKKwIDAQAB\n-----END PUBLIC KEY-----\n'


def encrypt2(message, pub_key, encode='utf8'):
    rsakey = RSA.importKey(pub_key)
    cipher = Cipher_pkcs1_v1_5.new(rsakey)
    cipher_text = base64.b64encode(cipher.encrypt(message))
    return cipher_text


def decrypt2(crypto, pri_key, encode='utf8'):
    rsakey = RSA.importKey(pri_key)
    cipher = Cipher_pkcs1_v1_5.new(rsakey)
    text = cipher.decrypt(base64.b64decode(crypto), None)
    return text.decode(encode)


#################################################################################################################################
# cryptography
#################################################################################################################################
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from oracle.database.dbs import get_soldier


def read_private_key(file, password=None):
    """
    加载私钥文件为对象
    :param file:
    :param password:
    :return:
    """
    with open(file, 'rb') as key_file:
        private_key = serialization.load_pem_private_key(
            key_file.read(),
            password=password,
            backend=default_backend()
        )
    return private_key


def read_public_key(file):
    """
    加载公钥文件为对象
    :param file:
    :return:
    """
    with open(file, 'rb') as key_file:
        public_key = serialization.load_pem_public_key(
            key_file.read(),
            backend=default_backend()
        )
    return public_key


def generate_private_key(public_exponent=65537, key_size=2048, password=None):
    """
    生成私钥字符串或者序列化对象
    :param public_exponent:
    :param key_size:
    :param password: 是否需要对序列化的输出加密
    :return:
    """
    private_key_obj = rsa.generate_private_key(
        public_exponent=public_exponent,
        key_size=key_size,
        backend=default_backend()
    )
    if password is None or len(password) == 0:
        encryption = serialization.NoEncryption()
    else:
        encryption = serialization.BestAvailableEncryption(password)
    private_key_str = private_key_obj.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=encryption
        # encryption_algorithm=serialization.BestAvailableEncryption(password)
    )
    return private_key_obj, private_key_str


def generate_public_key(private_key_obj):
    """
    根据私钥对象生成公钥串
    :param private_key_obj: cryptography类库的私钥对象
    :return:
    """
    public_key_obj = private_key_obj.public_key()
    public_key_str = public_key_obj.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo)
    return public_key_obj, public_key_str


def encrypt_rsa(message, public_key_obj, encode='utf8'):
    """
    使用公钥加密字符串
    :param message: 需要加密的字符串
    :param public_key_obj: cryptography类库的公钥对象
    :param encode: message字符串的编码格式
    :return: base64编码的字符串
    """
    cipher_text = public_key_obj.encrypt(
        # message,
        message.encode(encode),
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )
    # print('加密数据： ', cipher_text)
    return base64.b64encode(cipher_text)
    # return cipher_text


def decrypt_rsa(crypto_message_base64, private_key_obj, encode='utf8'):
    """
    对加密的字符串使用私钥解密
    :param crypto_message_base64 加密后的字符串 (进行了base64编码)
    :param private_key_obj: cryptography类库的私钥对象
    :param encode:
    :return: bytes
    """
    plain_text = private_key_obj.decrypt(
        # 加密的信息
        # crypto_message,
        base64.b64decode(crypto_message_base64),
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )
    # print('解密数据: ', plain_text)
    return plain_text


def encrypt_ecies(public_key, original_text, iv=None):
    """
    使用命令行调用楚学文的ECIES库
    crypto-tx -h
    Usage:
      crypto-tx -G [-json]
      crypto-tx -E [-k privateKey] -p publicKeyTo -d originaltext [-iv value] [-json]
      crypto-tx -D -k privateKey -p ephemPublicKey -d ciphertext [-iv value]
    Options:
      -E   cmd encryptECIES [no]
      -D   cmd decryptECIES [no]
      -G   cmd gen private and public keys [no]
      -k   privateKey hex
      -p   publicKey hex
      -d   encrypt or decrypt data
      -iv  IV 16 bytes hex
      -json convert json [no]
    :param public_key: 公钥
    :param original_text: 需要加密的字符串
    :param iv: 向量，可以缺省
    :return: (ciphertext, ephemPublicKey, iv, mac)
    """
    cmd = 'crypto-tx -E -p {public_key} -d {original_text} -json'.format(public_key=public_key, original_text=original_text)
    if iv is not None:
        cmd = ''.join([cmd, ' -iv {}'.format(iv)])
    args = shlex.split(cmd)
    return_json = subprocess.check_output(args, shell=False)
    return return_json


def decrypt_ecies(private_key, ephemPublicKey, ciphertext, iv=None):
    """
    ECIES 方式 解码
    crypto-tx -h
    Usage:
      crypto-tx -G [-json]
      crypto-tx -E [-k privateKey] -p publicKeyTo -d originaltext [-iv value] [-json]
      crypto-tx -D -k privateKey -p ephemPublicKey -d ciphertext [-iv value]
    Options:
      -E   cmd encryptECIES [no]
      -D   cmd decryptECIES [no]
      -G   cmd gen private and public keys [no]
      -k   privateKey hex
      -p   publicKey hex
      -d   encrypt or decrypt data
      -iv  IV 16 bytes hex
      -json convert json [no]
    :param private_key: 解密者的私钥
    :param ephemPublicKey: 加密者的公钥
    :param ciphertext: 密文
    :param iv: 加密向量
    :return: 解密后的数据
    """
    cmd = 'crypto-tx -D -k {private_key} -p {ephemPublicKey} -d {ciphertext} -iv {iv} '.format(private_key=private_key,
                                                                                             ephemPublicKey=ephemPublicKey,
                                                                                             ciphertext=ciphertext,
                                                                                             iv=iv)
    print(cmd)
    args = shlex.split(cmd)
    original_text = subprocess.check_output(args, shell=False)
    # TODO: 通过获得命令返回的状态来判断是否成功，目前crypto-tx不支持
    # original_text, tf = subprocess.getstatusoutput(cmd)
    return original_text.strip()


def check_ecies(private_key, ephem_public_key, ciphertext, verified_text, iv=None):
    """
    验证解码后的字符串是否相等
    :param private_key: 解密人的私钥
    :param ephem_public_key: 加密这个字符串的人的公钥 （使用私钥加密，必须把他私钥对应的公钥也传递过来）
    :param ciphertext: 加密后的字符串
    :param verified_text: 需要验证的字符串
    :param iv: 向量
    :return:
    """
    obj = decrypt_ecies(private_key=private_key, ephemPublicKey=ephem_public_key, ciphertext=ciphertext, iv=iv).strip()
    no_0x_bytes = obj.decode().lstrip('0x')
    return_str = bytes.fromhex(no_0x_bytes).decode()
    if return_str == verified_text:
        return True
    else:
        return False


def check_ecies_in_database(private_key, ephem_public_key, ciphertext, iv=None):
    """
    在数据库中匹配是否有这个加密的数据库
    :param private_key: 解密人的私钥
    :param ephem_public_key: 加密这个字符串的人的公钥 （使用私钥加密，必须把他私钥对应的公钥也传递过来）
    :param ciphertext: 加密后的字符串
    :param iv: 向量
    :return:
    """
    original = decrypt_ecies(private_key=private_key, ephemPublicKey=ephem_public_key, ciphertext=ciphertext, iv=iv).strip()
    no_0x_bytes = original.lstrip(b'0x').decode()
    # return_str = bytes.fromhex(no_0x_bytes).decode('utf8')
    # 在数据库中查询
    soldier = get_soldier(no_0x_bytes)
    if soldier is None:
        return False, None
    else:
        return True, soldier


def get_iv(value: str) -> str:
    """
    获得value对应的iv值
    :param value:
    :return:
    """
    from pycontractsdk import keys
    value_bytes = keys.keccak(text=value)
    keccak = value_bytes.hex()[0:32]
    return ''.join(['0x',keccak])



#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""
from oracle.dids import Did
import json


def test():
    did_json_str = """{"@context":"https://w3id.org/did/v1","id":"did:ethr:b2be7c8722935db16921d64ad0ddb5d47a52482a#soldier","publicKey":[{"id":"did:ethr:b2be7c8722935db16921d64ad0ddb5d47a52482a##proof","type":"EciesVerificationKey","controller":"did:ethr:b2be7c8722935db16921d64ad0ddb5d47a52482a","publicKeyHex":"0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297"}],"authentication":[{"id":"did:ethr:decd6626f23bc43cb4d738cb0f2dde922d691fd2#proof","type":"EciesVerificationKey","controller":"did:ethr:decd6626f23bc43cb4d738cb0f2dde922d691fd2","publicKeyHex":"0x024af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8"}],"proof":"00a991c5346ffd5763c851610ef95254f2ec37a0b5d88f8f55d778411b29b1062db7485674c69003720077d07701a37c","created":1554035188114}"""
    sign_text = """f4e085e52a799d062ad35e66c5ce3c842bab6bf5168e1860c187b46d23b6c8053e1df712bcb2e4c4a5901c93c3c17842835092a503dd4e8c39a4c344bd4832b701"""
    # did_json_obj = json.loads(did_json_str)
    tf = Did.verify_did(did_json_str, sign_text)
    print(tf)
    private_key = '0x6a6d5f34772ef3ba498c5256aa8470d2401e82a5921cbef58ec0c52e4336b925'
    sign_msg = Did.sign_did(did_json_str, sign_text, private_key)
    print(sign_msg)


def test_decrypt():
    """ 测试加密解密 """
    str = '0x00a991c5346ffd5763c851610ef95254f2ec37a0b5d88f8f55d778411b29b1062db7485674c69003720077d07701a37c'
    publicKeyHex = '0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297'
    privateKey = '0x6a6d5f34772ef3ba498c5256aa8470d2401e82a5921cbef58ec0c52e4336b925'
    from oracle.crypto import decrypt_ecies
    tf = decrypt_ecies(private_key=privateKey, ephemPublicKey=publicKeyHex, ciphertext=str)
    print(tf.lstrip(b'0x').decode())
    print(bytes.fromhex(tf.lstrip(b'0x')).decode())
    print(tf)


# 指定参数和返回类型
def say_hi(name: str = 'yyp') -> str:
    return "Hi," + name


if __name__ == '__main__':
    test()
    # from oracle.database.dbs import query_keys, db, query_contract
    # test_decrypt()









#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""
import datetime
from oracle import tasks
from celery import chain

def test_celery():
    # a = tasks.test2.apply_async((2,), link=tasks.test.si(2))
    # print(a.get())
    tk = tasks.add.apply_async((2,2), link=tasks.add.s(3))
    print(tk.get())
    # res = chain(tasks.add.s(2,2), tasks.add.s(2))
    # print(res.get())




if __name__ == '__main__':
    # # 修改celery的配置
    # from oracle.tasks import app
    # app.conf.update(
    #     result_backend='redis://127.0.0.1:6379/1',
    #     broker_url='redis://127.0.0.1:6379/1',
    # )
    # test_celery()
    # # app.star()

    test_iv()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

PATH = ''
# 加密解密的向量
IV = b'2WDFR5az'
# 开始 private key 的加密和解密
ENCRYPTO = False

VAR = {
    # 存放了相关的配置信息
    'MYSQL_TABLE_PRE': 'cp',
    'WEIXIN_MSG_TMP': '用户:{} 身份:{} 验证成功'
}



import configparser
import json
import os
from oracle import utils


def read_config(file='/etc/oracle.conf'):
    """
    读取配置文件的信息到 全局变量
    :param file: 配置文件的路径
    :return:
    """
    # 声明全局变量，才能修改数据
    global ETH_PORT, ETH_IP, ETH_URL, ETH_TIMEOUT
    global MYSQL_USER, MYSQL_PASSWORD, MYSQL_SCHEMA, MYSQL_PORT, MYSQL_IP, MYSQL_CHARSET

    config = configparser.RawConfigParser()
    config.read(file, encoding='utf8')
    # 读取 section=mysql 的配置信息
    MYSQL_USER = config.get('mysql', 'user')
    MYSQL_PASSWORD = config.get('mysql', 'password')
    MYSQL_SCHEMA = config.get('mysql', 'schema')
    MYSQL_PORT = config.getint('mysql', 'port')
    MYSQL_IP = config.get('mysql', 'ip')
    MYSQL_CHARSET = config.get('mysql', 'charset')

    # 读取 section=eth 的配置细信息
    ETH_PORT = config.getint('eth', 'port')
    ETH_IP = config.get('eth', 'ip')
    ETH_TIMEOUT = config.getint('eth', 'timeout')

    # 读取机器相关的配置信息
    MACHINE_FILE = config.get('common', 'machine_file')


def read_config2(file='/etc/oracle.conf'):
    """
    读取配置文件的信息到 VAR 变量
    :param file: 配置文件的路径
    :return:
    """
    config = configparser.RawConfigParser()
    config.read(file, encoding='utf8')
    global PATH
    PATH=os.path.split(os.path.abspath(os.path.basename(__file__)))[0]
    # pas = utils.DESEncrypt('hello world')
    # mw = utils.DESDecrypt('ddd'+pas+"s")
    # print(mw)
    # exit(0)

    # 读取 section=mysql 的配置信息
    VAR['MYSQL_USER'] = config.get('mysql', 'user')
    VAR['MYSQL_PASSWORD'] = config.get('mysql', 'password')
    VAR['MYSQL_SCHEMA'] = config.get('mysql', 'schema')
    VAR['MYSQL_PORT'] = config.getint('mysql', 'port')
    VAR['MYSQL_IP'] = config.get('mysql', 'ip')
    VAR['MYSQL_CHARSET'] = config.get('mysql', 'charset')
    VAR['MAX_CONNECTION'] = config.getint('mysql', 'max_connection')
    VAR['STALE_TIMEOUT'] = config.getint('mysql', 'stale_timeout')

    # 读取 机器相关的信息
    VAR['MACHINE_FILE'] = config.get('common', 'machine_file')
    VAR['MACHINE_ID'] = utils.get_machine_key(VAR['MACHINE_FILE'])

    # 初始化数据库，并且从数据库中获得private key信息
    from oracle.database.modles import db
    db.init(
        VAR['MYSQL_SCHEMA'],
        host=VAR['MYSQL_IP'],
        user=VAR['MYSQL_USER'],
        password=VAR['MYSQL_PASSWORD'],
        charset=VAR['MYSQL_CHARSET'],
        max_connections=VAR['MAX_CONNECTION'],
        stale_timeout=VAR['STALE_TIMEOUT'])
    from oracle.database.dbs import query_keys, query_contract
    key = query_keys(VAR['MACHINE_ID'])
    VAR['USER_PRIVATE_KEY'] = utils.DESDecrypt(key.private_key) if ENCRYPTO else key.private_key
    VAR['USER_ADDRESS'] = key.address

    contract = query_contract()
    VAR['CONTRACT_ADDRESS'] = contract.contract_address
    # VAR['CONTRACT_ABI'] = contract.abi
    json_obj = json.loads(contract.abi)
    VAR['CONTRACT_ABI'] = json_obj['abi']

    # 读取 section=eth 的配置信息
    VAR['ETH_PROVIDER'] = config.get('eth', 'provider')
    VAR['ETH_TIMEOUT'] = config.getint('eth', 'timeout')
    VAR['CONTRACT_ADDRESS'] = config.get('eth', 'address')
    VAR['ARMY_PRIVATEKEY'] = utils.DESDecrypt(config.get('eth', 'army_privatekey')) if ENCRYPTO else config.get('eth', 'army_privatekey')
    VAR['OPERATOR_PRIVATEKEY'] = utils.DESDecrypt(config.get('eth', 'operator_privatekey')) if ENCRYPTO else config.get('eth', 'operator_privatekey')
    VAR['DELEGATE_PRIVATEKEY'] = utils.DESDecrypt(config.get('eth', 'delegate_privatekey')) if ENCRYPTO else config.get('eth', 'delegate_privatekey')
    VAR['GAS'] = config.get('eth', 'gas')
    VAR['GAS_PRISE'] = config.get('eth', 'gas_prise')


    # 读取 section=common 的配置信息
    VAR['BLOCKNUM_CACHE_FILE'] = config.get('common', 'blocknum_cache_file')
    VAR['ARMY_BLOCKNUM_CACHE_FILE'] = config.get('common', 'army_blocknum_cache_file')
    VAR['ORGANIZATION_BLOCKNUM_CACHE_FILE'] = config.get('common', 'organization_blocknum_cache_file')
    VAR['ContractOwnerChange'] = config.get('common', 'ContractOwnerChange')
    VAR['DIDOwnerChanged'] = config.get('common', 'DIDOwnerChanged')
    VAR['DIDDelegateChanged'] = config.get('common', 'DIDDelegateChanged')
    VAR['DIDAttributeChange'] = config.get('common', 'DIDAttributeChange')
    VAR['DIDAttributeConfirmed'] = config.get('common', 'DIDAttributeConfirmed')
    VAR['BLOCKNUM_INC'] = config.getint('common', 'blocknum_inc')
    VAR['IDENTITY'] = config.get('common', 'identity')


    # 读取 section=redis 的配置信息
    VAR['REDIS_IP'] = config.get('redis', 'ip')
    VAR['REDIS_PORT'] = config.get('redis', 'port')
    VAR['REDIS_DATABASE'] = config.get('redis', 'database')
    VAR['REDIS'] = 'redis://{}:{}/1'.format(VAR['REDIS_IP'], VAR['REDIS_PORT'])

    # # 修改celery的配置
    # from oracle.tasks import app
    # app.conf.update(
    #     result_backend=VAR['REDIS'],
    #     broker_url=VAR['REDIS'],
    # )

    # 读取 section=wechat 的配置信息
    VAR['WECHAT_APPID'] = config.get('wechat', 'appid')
    VAR['WECHAT_SECRET'] = config.get('wechat', 'secret')


    # 读取 section=rabbitmq 的配置信息
    VAR['QUEUE_NAME'] = config.get('rabbitmq', 'queue_name')
    VAR['QUEUE_NAME_VALIDATION'] = config.get('rabbitmq', 'queue_name_validation')
    VAR['QUEUE_NAME_NOTICE'] = config.get('rabbitmq', 'queue_name_notice')
    VAR['QUEUE_IP'] = config.get('rabbitmq', 'queue_ip')
    VAR['QUEUE_PORT'] = config.get('rabbitmq', 'queue_port')
    VAR['QUEUE_USER'] = config.get('rabbitmq', 'queue_user')
    VAR['QUEUE_PASSWORD'] = config.get('rabbitmq', 'queue_password')
    VAR['QUEUE_VHOST'] = config.get('rabbitmq', 'queue_vhost')


    # 读取 section=kafka 的配置信息
    VAR['KAFKA_IP'] = config.get('kafka', 'kafka_ip')
    VAR['KAFKA_PORT'] = config.get('kafka', 'kafka_port')
    VAR['KAFKA_TOPIC'] = config.get('kafka', 'kafka_topic')
    VAR['KAFKA_TOPIC_VALIDATION'] = config.get('kafka', 'kafka_topic_validation')
    VAR['KAFKA_TOPIC_NOTICE'] = config.get('kafka', 'kafka_topic_notice')


    # 读取 section=mail 的配置信息
    VAR['MAIL_SMTP'] = config.get('mail', 'smtp')
    VAR['MAIL_USER'] = config.get('mail', 'user')
    VAR['MAIL_PASSWORD'] = config.get('mail', 'password')
    VAR['MAIL_PORT'] = config.get('mail', 'port')







#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""


import asyncio
import sys

@asyncio.coroutine
def first_coroutine(future, N):
    """前n个数的和"""
    count = 0
    for i in range(1, N + 1):
        count = count + i
    yield from asyncio.sleep(4)
    future.set_result("first coroutine (sum of N integers) result = " + str(count))

@asyncio.coroutine
def second_coroutine(future, N):
    count = 1
    for i in range(2, N + 1):
        count *= i
    yield from asyncio.sleep(3)
    future.set_result("second coroutine (factorial) result = " + str(count))

def got_result(future):
   print(future.result())

if __name__ == "__main__":
   N1 = int(sys.argv[1])
   N2 = int(sys.argv[2])
   loop = asyncio.get_event_loop()
# future = loop.create_future()
   future1 = asyncio.Future()
   future2 = asyncio.Future()
   tasks = [  # 有2个任务
       first_coroutine(future1, N1),
       second_coroutine(future2, N2)]
   future1.add_done_callback(got_result)  # 完成任务的回调
   future2.add_done_callback(got_result)
   loop.run_until_complete(asyncio.wait(tasks))
   loop.close()


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""



from oracle.celery.task import *

from celery import chain

res = taskA.apply_async((1,2), link=taskB.s(1,2))

# res = chain(taskA.s(1,2), taskB.s(1,1))
print(res.get())

# re1 = taskA.delay(100, 200)
# # print(re1.get())
# re2 = taskB.delay(1, 2, 3)
# # print(re2.result)
# re3 = add.delay(1, 2)
# # print(re3.status)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
工具类库
"""

import os.path
from oracle.exceptions import ValidationError
import random2
import json
import re
import ctypes
from ctypes import CDLL
from oracle import setting
from oracle.logger import logger

def create_machine_key(absfile):
    """
    生成唯一的机器标识符
    :param file: 文件的绝对路径
    :return:
    """
    if os.path.isdir(absfile):
        raise ValueError("%s must be a file", absfile)
    (path, file) = os.path.split(absfile)
    if not os.path.exists(path):
        os.makedirs(path, mode=0o777, exist_ok=False)
    with open(absfile, 'w', encoding='utf8') as f:
        rand = round(random2.SystemRandom().random() * 10000000000000000)
        f.write(str(rand))
    return rand


def get_machine_key(absfile):
    """
    获得本地的机器唯一标识符
    :param file: 文件的绝对路径
    :return: 机器标识符
    """
    if not os.path.exists(absfile):
        return create_machine_key(absfile)
    with open(absfile, 'r') as f:
        machine = f.read()
        if machine.strip().strip('\n').strip('\t') == '':
            machine = create_machine_key(absfile)
    return machine

def get_abi(abifile):
    """
    从文件中获得abi对象
    :param abifile: 本地的abi文件
    :return:
    """
    if not os.path.isfile(abifile):
        raise ValidationError("%s file is not exist" % abifile)
    with open(abifile) as f:
        json_str_data = f.read()
    json_obj = json.loads(json_str_data)
    return json_obj['abi']


def write_blocknum(cache_file, block_num):
    """
    将块号写入指定的文件
    :param cache_file:
    :param block_num:
    :return:
    """
    with open(cache_file, 'w') as f:
        f.write(str(block_num))


def get_cache_blocknum(cache_file):
    """
    获得缓存文件中的块号，如果不存在就返回0，并且新建文件
    :param cache_file:
    :return:
    """
    if os.path.isfile(cache_file):
        with open(cache_file) as f:
            block_num = f.read()
            num = block_num.strip().strip('\n').strip('\t')
            if re.match('^[0-9]+$', num):
                return int(num)
            else:
                raise ValueError('block number: %s is not number' % block_num)
    else:
        (path, file) = os.path.split(cache_file)
        if not os.path.exists(path):
            os.makedirs(path, mode=0o777, exist_ok=False)
        with open(cache_file, 'w', encoding='utf8') as f:
            f.write(str(0))
        return 0


def import_plugins():
    """ 用于加载`plugin` """
    import importlib
    here = os.path.abspath(os.path.dirname(__file__))
    path = '{}/plugins/'.format(here)
    for root, dirs, files in os.walk(path):
        if root == path:
            for file in files:
                if file != '__init__.py' and re.match(r'^(.*)\.py$', file):
                    name = file.split('.')[0]
                    importlib.import_module('oracle.plugins.{}'.format(name))
    # importlib.import_module('oracle.plugins')
    # importlib.import_module('oracle.plugins.events')
    # importlib.import_module('oracle.plugins.solider')


def DESEncrypt(txt):
    """
    加密数据
    :param txt: 明文
    :return:
    """
    DESEncrypt = CDLL('{}/../golang/crypto.so'.format(setting.PATH)).DESEncrypt  # 调用ｇｏ模块
    # 显式声明参数和返回的期望类型
    DESEncrypt.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
    DESEncrypt.restype = ctypes.c_char_p
    pas = DESEncrypt(txt.encode(), setting.IV)
    return pas.decode()


def DESDecrypt(ciphertext):
    """
    解密数据
    :param ciphertext: 密文
    :return:
    """
    DESDecrypt = CDLL('{}/../golang/crypto.so'.format(setting.PATH)).DESDecrypt  # 调用ｇｏ模块
    # 显式声明参数和返回的期望类型
    DESDecrypt.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
    DESDecrypt.restype = ctypes.c_char_p
    mw = DESDecrypt(ciphertext.encode(), setting.IV)
    return mw.decode()


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""

# cp_army 数据库表的状态
EVENT_NORMAL = 0                        # 初始状态
EVENT_SIGN_VERI_FAILED = 1              # 签名验证失败
EVENT_SIGN_VERI_SUCCESS = 2             # 签名验证成功
EVENT_DECRYPTION_FAILED = 3             # 密文解密失败
EVENT_DECRYPTION_SUCCESS = 4            # 密文解密成功
EVENT_ETHEREUM_SUBMIT_FAILED = 5        # 发送到以太坊失败
EVENT_ETHEREUM_SUBMIT_SUCCESS = 6       # 发送到以太坊成功
EVENT_ETHEREUM_SOLDIER_NOTEXIST = 7     # 士兵不存在
EVENT_VERI_COMP = 8                     # 数据验证完成（到这一步就可以进行上链操作了）

# cp_organization 表的状态
ORG_NORMAL = 0                          # 初始状态
ORG_FIRST_SIGN_VERI_FAILED = 1          # 验证第一个签名失败
ORG_FIRST_SIGN_VERI_SUCCESS = 2         # 验证第一个签名成功
ORG_SECOND_SIGN_VERI_FAILED = 3         # 验证第二个签名失败
ORG_SECOND_SIGN_VERI_SUCCESS = 4        # 验证第二个签名成功
ORG_SEND_WECHAT_SUCCESS = 5             # 微信消息发送成功
ORG_SEND_WECHAT_FAILED = 6              # 微信消息发送失败

# person_dc 表的状态
DC_NORMAL = 0                           # 初始状态
DC_UP_CHAIN_TIMEOUT = 1                 # 上链超时
DC_UP_CHAIN_FAILED = 2                  # 上链失败
DC_UP_CHAIN_SUCCESS = 3                 # 上链成功






#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-17 12:37:21
LastEditors:  
LastEditTime: 2019-04-17 12:37:21
Description:  
"""
from oracle.logger import logger
import toolz

class Processor(object):
    # 注册的组件
    """
    PLUGINS = {
        'event_handle': {
            'DIDAttributeChange': CLASS,    // 处理指定`event`的class
            'DIDAttributeConfirmed': CLASS
        },
        'identity': {
            'DIDAttributeChange': {
                '士兵': 'Class',    // 不同的身份不同的处理函数
                'soldier': 'Class'
            },
            'DIDAttributeConfirmed': {
                '士兵': 'Class',
                'soldier': 'Class'
            }
        }
    }
    """
    PLUGINS = {}

    def process_identity(self, event, identity, entry):
        if identity not in toolz.get_in(['identity', event], self.PLUGINS):
            logger.error("{} 这个插件不存在, 请检查代码".format(identity))
        else:
            if hasattr(self.PLUGINS['identity'][event][identity], 'before'):
                self.PLUGINS['identity'][event][identity]().before(entry)

            self.PLUGINS['identity'][event][identity]().process(entry)

            if hasattr(self.PLUGINS['identity'][event][identity], 'after'):
                self.PLUGINS['identity'][event][identity]().after(entry)

    def process_event(self, event, entry):
        if 'event_handle' not in self.PLUGINS:
            logger.error("系统目前没有任何`event`插件, 请检查代码".format(event))
        else:
            if event not in self.PLUGINS['event_handle']:
                logger.error("{} 这个插件不存在, 请检查代码".format(event))
            else:
                if hasattr(self.PLUGINS['event_handle'][event], 'before'):
                    self.PLUGINS['event_handle'][event]().before(entry)

                self.PLUGINS['event_handle'][event]().process(entry)

                if hasattr(self.PLUGINS['event_handle'][event], 'after'):
                    self.PLUGINS['event_handle'][event]().after(entry)

        pass

    @classmethod
    def plugin_register(cls, event=None, identitys=[]):
        """
        注册插件的包装器
        如果`identitys`没有数据，注册的就是对应`event`事件的插件，用于处理对应的event的数据
        如果`identitys`有数据，注册的就是`event`中具体身份的处理函数
        :param event:
        :param identitys:
        :return:
        """
        if event is None:
            print('注册的event不能为空')
            exit(2)
        def wrapper(plugin):
            if len(identitys) == 0:
                if 'event_handle' not in cls.PLUGINS:
                    cls.PLUGINS.update({'event_handle': {event: plugin}})
                else:
                    cls.PLUGINS['event_handle'].update({event: plugin})
            for identity in identitys:
                if 'identity' not in cls.PLUGINS:
                    cls.PLUGINS.update({'identity': {event: {identity: plugin}}})
                else:
                    cls.PLUGINS = toolz.assoc_in(cls.PLUGINS, ['identity', event, identity], plugin)
            return plugin
        return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# from oracle.database.db import *


# def test():
#     logger.info("xxxxxxx")


if __name__ == "__main__":
    # test()
    # logger.info("my test.")
    # logger.error("my test.")
    # create_person(name='yuyongpeng')
    # query_person(name='cat')
    # key = query_keys('6bS8KmBtE1Rs5Eg')
    # logger.info(key.address)

    str1 = 'testing '
    str2 = 'string '
    str3 = 'concation '
    # str = ''.join([str1, str2, str3])
    # print(str)



class Generator(object):

    def __init__(self, name):
        self.name = name

    def __enter__(self):
        print("There's no way out")
        return self

    def __exit__(self, exc_type, exc_value, traceback):
          # 检查退出的类型
        # 当有异常发生时 exc_type存在值
        if exc_type:
            print('Open the door')
        else:
            print('100%')

    def generate(self):
        print('%s can uses love to generate electricity' % self.name)

# 紧跟with后面的语句被求值后，返回对象的 `__enter__() `方法被调用,
# 这个方法的返回值将被赋值给as后面的变量
with Generator("LEX") as lex:
    lex.generate()
# 当with后面的代码块全部被执行完之后，将调用返回对象的 `__exit__()`方法。

from contextlib import contextmanager
@contextmanager
def opening(filename):
    f = open(filename)  # IOError is untouched by GeneratorContext
    try:
        yield f
    finally:
        f.close()  # Ditto for errors here (however unlikely)

class MyFunc(object):
    """实现上下文管理器"""

    def __init__(self, file_name, mode):
        self.file_name = file_name
        self.mode = mode

    def __enter__(self):
        self.f_obj = open(self.file_name, self.mode)
        return self.f_obj          # __enter__函数返回的是打开资源的对象

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.f_obj.close()


with open("file_name", "r") as file_obj:
    content = file_obj.read()

print(content)

from datetime import datetime
unix_ts = 1509636585.0
times = datetime.fromtimestamp(unix_ts)
a = times.strftime("%Y-%m-%d %H:%M:%S")


# -*- coding: utf-8 -*-

import os
import logbook
from logbook import Logger, TimedRotatingFileHandler
from logbook.more import ColorizedStderrHandler

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S.00"


def user_handler_log_formatter(record, handler):
    log = "[{dt}][{level}][{filename}][{func_name}][{lineno}] {msg}".format(
        dt=record.time,                                 # 时间
        level=record.level_name,                        # 日志等级
        filename=os.path.split(record.filename)[-1],    # 文件名
        func_name=record.func_name,                     # 函数名
        lineno=record.lineno,                           # 行号
        msg=record.message,                             # 日志内容
    )
    return log


# 日志路径，在主工程下生成log目录
LOG_DIR = os.path.join('log')
if not os.path.exists(LOG_DIR):
    os.makedirs(LOG_DIR)

# 打印到屏幕句柄
user_std_handler = ColorizedStderrHandler(bubble=True)
user_std_handler.formatter = user_handler_log_formatter

# 打印到文件句柄 [每天一个文件，备份7天的数据]
user_file_handler = TimedRotatingFileHandler(
    os.path.join(LOG_DIR, '%s.log' % 'oracle'), date_format='%Y%m%d', backup_count=7, bubble=True)
user_file_handler.formatter = user_handler_log_formatter

# 用户代码logger日志
logger = Logger("oracle_log")


def init_logger():
    logbook.set_datetime_format("local")        # 使用本地时间
    logger.handlers.append(user_std_handler)    # 添加命令行输出
    logger.handlers.append(user_file_handler)   # 添加文件输出
    logger.level = logbook.DEBUG              # log的级别


init_logger()



# -*- coding: utf-8 -*-

"""
存放的是数据库的模型
"""

# from peewee import *
from peewee import ForeignKeyField, Model, CharField, DateTimeField, IntegerField, TimestampField, AutoField
from oracle.setting import VAR
from playhouse.pool import PooledMySQLDatabase
import datetime

# 将peewee的操作输出详细的日志
import logging

logger = logging.getLogger('peewee')
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)

# url: mysql+pool://root:root@127.0.0.1:3306/appmanage?max_connections=300&stale_timeout=300
# db = connect(url=setting.MYSQL_URL)

# db = MySQLConnectorDatabase(
#     setting.MYSQL_SCHEMA,
#     host=setting.MYSQL_IP,
#     user=setting.MYSQL_USER,
#     password=setting.MYSQL_PASSWORD,
#     charset=setting.MYSQL_CHARSET )

# db = PooledMySQLDatabase(
#     setting.VAR['MYSQL_SCHEMA'],
#     host=setting.VAR['MYSQL_IP'],
#     user=setting.VAR['MYSQL_USER'],
#     password=setting.VAR['MYSQL_PASSWORD'],
#     charset=setting.VAR['MYSQL_CHARSET'],
#     max_connections=8,
#     stale_timeout=300)

# 放到后面通过 db.init()方式来初始化连接
# https://peewee.readthedocs.io/en/latest/peewee/database.html#run-time-database-configuration
db = PooledMySQLDatabase(None)


def make_table_name(model_class):
    """ 指定表的前缀 """
    model_name = model_class.__name__
    return VAR['MYSQL_TABLE_PRE'] + '_' + model_name.lower()


class BaseModel(Model):
    """A base model that will use our MySQL database"""

    class Meta:
        database = db
        table_function = make_table_name


class Soldiers(BaseModel):
    """
    退伍军人的hash信息
    """
    encrypted_string = CharField(column_name='encrypted_string', help_text='身份加密后的信息')
    verification_num = IntegerField(column_name='verification_num', help_text='验证的次数（目前没有使用）')
    update_time = DateTimeField(column_name='update_time', default=datetime.datetime.now, help_text='数据更新时间')
    create_time = DateTimeField(column_name='create_time', help_text='数据建立时间')


class Keys(BaseModel):
    """
    用于加密解密和签名的公钥，私钥信息
    """
    private_key = CharField(column_name='private_key', help_text='账号的私钥')
    public_key = CharField(column_name='public_key', help_text='账号对应的公钥')
    address = CharField(column_name='address', help_text='账号的address')
    status = IntegerField(column_name='status', help_text='账号的状态 数据的状态 0: 正常，1:删除')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now, help_text='数据的更新时间')
    machine = CharField(column_name='machine', help_text='机器的唯一标识符，程序启动的时候会固化')


class Contract(BaseModel):
    """
    记录了智能协约的address和abi
    """
    contract_address = CharField(column_name='contract_address')
    abi = CharField(column_name='abi')
    status = IntegerField(column_name='status')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now)


class Verification(BaseModel):
    """
    记录了events的数据，用于验证和签名
    """
    event = CharField(column_name='event', help_text='event名称')
    mechanism = CharField(column_name='mechanism', help_text='机构')
    owner_address = CharField(column_name='owner_address', help_text='所有者的address')
    identity_address = CharField(column_name='identity', help_text='用户的address')
    name = CharField(column_name='name', help_text='属性名')
    value = CharField(column_name='value', help_text='属性值')
    sign_text = CharField(column_name='sign_text', help_text='DID的签名数据（R,S,V）')
    scrpto_text = CharField(column_name='scrpto_text', help_text='公钥签名的数据')
    user_id = IntegerField(column_name='user_id', help_text='用户表的id，通过这个查找微信用户的OpenID')
    status = IntegerField(column_name='status', help_text='0:未验证，1:已验证，2:加密后发送')
    generation = CharField(column_name='generation', help_text='数据导入的时间(不会变更)')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now)


class Events(BaseModel):
    """
    记录了所有的日志信息
    CREATE TABLE `cp_events` (
      `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键',
      `args` mediumtext COMMENT 'event的参数',
      `event_name` varchar(200) DEFAULT NULL COMMENT 'event的名称',
      `log_index` int(10) DEFAULT NULL COMMENT 'log的索引号',
      `transaction_index` int(10) DEFAULT NULL COMMENT '交易的索引号',
      `transaction_hash` varchar(200) DEFAULT NULL COMMENT '交易的hash',
      `address` varchar(200) DEFAULT NULL COMMENT '所属智能协约的地址',
      `block_hash` varchar(200) DEFAULT NULL COMMENT '所属块的hash',
      `block_number` int(10) DEFAULT NULL COMMENT '所属的块号',
      `generation` datetime DEFAULT NULL COMMENT '数据生成时间',
      `updates` datetime DEFAULT NULL COMMENT '更新时间',
      `first_sign` mediumtext COMMENT '第一次签名的数据',
      `second_sign` mediumtext COMMENT '第二次签名的数据',
      `did` mediumtext COMMENT 'did 数据',
      PRIMARY KEY (`id`),
      UNIQUE KEY `tx` (`event_name`,`transaction_hash`,`transaction_index`) USING BTREE COMMENT '标识唯一的一个event'
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='存放监听到的所有event数据'
    """
    args = CharField(column_name='args', help_text='event的参数')
    event_name = CharField(column_name='event_name', help_text='event的名称')
    log_index = IntegerField(column_name='log_index', help_text='log的索引')
    transaction_index = IntegerField(column_name='transaction_index', help_text='交易的索引号')
    transaction_hash = CharField(column_name='transaction_hash', help_text='交易的hash值')
    address = CharField(column_name='address', help_text='event所属的智能协约地址')
    block_hash = CharField(column_name='block_hash', help_text='所属块的hash')
    block_number = IntegerField(column_name='block_number', help_text='所属的块号')
    generation = CharField(column_name='generation', default=datetime.datetime.now, help_text='数据导入的时间(不会变更)')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now, help_text='数据更新时间')
    first_sign = CharField(column_name='first_sign', help_text='第一次签名的数据')
    second_sign = CharField(column_name='second_sign', help_text='第二次签名的数据')
    did = CharField(column_name='did', help_text='did 数据')


class Army(BaseModel):
    """
    记录了 军队 验证event的数据流水
    """
    args = CharField(column_name='args', help_text='event的参数')
    event_name = CharField(column_name='event_name', help_text='event的名称')
    log_index = IntegerField(column_name='log_index', help_text='log的索引')
    transaction_index = IntegerField(column_name='transaction_index', help_text='交易的索引号')
    transaction_hash = CharField(column_name='transaction_hash', help_text='交易的hash值')
    address = CharField(column_name='address', help_text='event所属的智能协约地址')
    block_hash = CharField(column_name='block_hash', help_text='所属块的hash')
    block_number = IntegerField(column_name='block_number', help_text='所属的块号')
    generation = CharField(column_name='generation', default=datetime.datetime.now, help_text='数据导入的时间(不会变更)')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now, help_text='数据更新时间')
    first_sign = CharField(column_name='first_sign', help_text='第一次签名的数据')
    second_sign = CharField(column_name='second_sign', help_text='第二次签名的数据')
    did_json = CharField(column_name='did_json', help_text='did 数据')
    created = IntegerField(column_name='created', help_text='did中的created时间 13位的时间戳，只处理时间比这个大的')
    created2 = DateTimeField(column_name='created2', help_text='did中的created时间转换为10位的时间戳')
    status = DateTimeField(column_name='status', help_text='数据的状态字段 0:正常, 1:名验证失败, 2:签名验证成功, 3:密文解密失败, 4:密文解密成功, 5:发送到以太坊失败, 6:发送到以太坊成功, 7:士兵不存在')
    did_id = CharField(column_name='did_id', help_text='存放当前用户的id（did串的id）')
    txhash = CharField(column_name='txhash', help_text='数据第二次上链后获得的交易hash')

class Organization(BaseModel):
    """
    记录了 机构 验证event的数据流水
    """
    args = CharField(column_name='args', help_text='event的参数')
    identity = CharField(column_name='identity', help_text='event中记录的identity字段的数据')
    event_name = CharField(column_name='event_name', help_text='event的名称')
    log_index = IntegerField(column_name='log_index', help_text='log的索引')
    transaction_index = IntegerField(column_name='transaction_index', help_text='交易的索引号')
    transaction_hash = CharField(column_name='transaction_hash', help_text='交易的hash值')
    address = CharField(column_name='address', help_text='event所属的智能协约地址')
    block_hash = CharField(column_name='block_hash', help_text='所属块的hash')
    block_number = IntegerField(column_name='block_number', help_text='所属的块号')
    generation = CharField(column_name='generation', default=datetime.datetime.now, help_text='数据导入的时间(不会变更)')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now, help_text='数据更新时间')
    first_sign = CharField(column_name='first_sign', help_text='第一次签名的数据')
    second_sign = CharField(column_name='second_sign', help_text='第二次签名的数据')
    did_json = CharField(column_name='did_json', help_text='did 数据')
    created = IntegerField(column_name='created', help_text='did中的created时间，只处理时间比这个大的')
    status = DateTimeField(column_name='status', help_text='数据的状态字段 0:正常, 1:验证第一个签名失败, 2:验证第一个签名成功, 3:验证第二个签名失败, 4:验证第二个签名成功')
    did_id = CharField(column_name='did_id', help_text='存放当前用户的id（did串的id）')
    mark = CharField(column_name='mark', help_text='人的身份标识（士兵，公务员，学生...）')


class OrganizationDid(BaseModel):
    """
    记录了 机构 验证event的数据流水
    """
    args = CharField(column_name='args', help_text='event的参数')
    identity = CharField(column_name='identity', help_text='event中记录的identity字段的数据')
    event_name = CharField(column_name='event_name', help_text='event的名称')
    log_index = IntegerField(column_name='log_index', help_text='log的索引')
    transaction_index = IntegerField(column_name='transaction_index', help_text='交易的索引号')
    transaction_hash = CharField(column_name='transaction_hash', help_text='交易的hash值')
    address = CharField(column_name='address', help_text='event所属的智能协约地址')
    block_hash = CharField(column_name='block_hash', help_text='所属块的hash')
    block_number = IntegerField(column_name='block_number', help_text='所属的块号')
    generation = CharField(column_name='generation', default=datetime.datetime.now, help_text='数据导入的时间(不会变更)')
    updates = DateTimeField(column_name='updates', default=datetime.datetime.now, help_text='数据更新时间')
    first_sign = CharField(column_name='first_sign', help_text='第一次签名的数据')
    second_sign = CharField(column_name='second_sign', help_text='第二次签名的数据')
    did_json = CharField(column_name='did_json', help_text='did 数据')
    created = DateTimeField(column_name='created', help_text='did中的created时间，只处理时间比这个大的')
    status = DateTimeField(column_name='status', help_text='数据的状态字段 0:正常')
    did_id = IntegerField(column_name='did_id', help_text='存放当前用户的id（did串的id）')
    mark = CharField(column_name='mark', help_text='人的身份标识（士兵，公务员，学生...）')


class Area(BaseModel):
    area_id = IntegerField(column_name='area_id', primary_key=True)
    area_province = CharField(column_name='area_province')
    area_city = CharField(column_name='area_city')
    modify_id = IntegerField(column_name='modify_id')
    modify_time = TimestampField(column_name='modify_time', help_text='')

    class Meta:
        table_name = 'area'


class Person(BaseModel):
    person_id = IntegerField(column_name='person_id', primary_key=True)
    address = CharField(column_name='address', help_text='')
    public_key = CharField(column_name='public_key', help_text='')
    name = CharField(column_name='name', help_text='')
    sex = IntegerField(column_name='sex', help_text='')
    age = IntegerField(column_name='age', help_text='')
    education = CharField(column_name='education', help_text='')
    working_experience = CharField(column_name='working_experience', help_text='')
    area = ForeignKeyField(Area, column_name='area_id', backref='person')
    modify_time = TimestampField(column_name='modify_time', help_text='')
    iphone = CharField(Area, column_name='iphone', help_text='')

    class Meta:
        table_name = 'person'


class PersonDc(BaseModel):
    id = IntegerField(column_name='id', primary_key=True)
    modify_time = TimestampField(column_name='modify_time', help_text='')
    value = CharField(column_name='value', help_text='')
    person_hash = CharField(column_name='person_hash', help_text='')
    organization_hash = CharField(column_name='organization_hash', help_text='')
    person_address = CharField(column_name='person_address', help_text='')
    organization_address = CharField(column_name='organization_address', help_text='')
    status = IntegerField(column_name='status', help_text='0:正常，1上链超时，2：上链失败，3：上链成功')
    tx_hash = CharField(column_name='tx_hash', help_text='交易的hash值')

    class Meta:
        table_name = 'person_dc'


class Account(BaseModel):
    """ 记录了用户的账号信息 """
    account_id = IntegerField(column_name='account_id', primary_key=True, help_text='id')
    weixin_openid = IntegerField(column_name='weixin_openid', help_text='微信的openid')
    weixin_unionid = IntegerField(column_name='weixin_unionid', help_text='weixin_unionid')
    iphone = IntegerField(column_name='iphone')
    modify_time = TimestampField(column_name='modify_time')
    # person_id = IntegerField(column_name='person_id')
    person = ForeignKeyField(Person, backref='account', column_name='person_id')

    class Meta:
        table_name = 'account'


class QueueFlow(BaseModel):
    """ 队列的流水表 """
    id = AutoField()
    type = CharField(column_name='type', help_text='[call_function|upload_chain] 类型')
    retry = IntegerField(column_name='retry', help_text='重试次数')
    upload_time = DateTimeField(column_name='upload_time', help_text='上链时间')
    status = IntegerField(column_name='status', help_text='0：初始状态，1：上链成功，2：上链失败')
    receipt = CharField(column_name='receipt', help_text='receipt hash')
    data = DateTimeField(column_name='data', help_text='队列中的json串')
    creates = DateTimeField(column_name='creates', default=datetime.datetime.now, help_text='数据插入到数据库的时间')
    updates = DateTimeField(column_name='updates', help_text='数据更新的时间')

    class Meta:
        table_name = 'queue_flow'






#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""

from Crypto.PublicKey import RSA


# def encrypt(message):
#     externKey = "/Users/yuyongpeng/git/hard-chain/cport/oracle/contracts/myPublicKey.pem"
#     privatekey = open(externKey, "r")
#     encryptor = RSA.importKey(privatekey, passphrase="f00bar")
#     encriptedData = encryptor.encrypt(message, 0)
#     file = open("/Users/yuyongpeng/git/hard-chain/cport/oracle/contracts/cryptThingy.txt", "wb")
#     file.write(encriptedData[0])
#     file.close()
#
# def decrypt():
#     externKey="/Users/yuyongpeng/git/hard-chain/cport/oracle/contracts//myPrivateKey.pem"
#     publickey = open(externKey, "r")
#     decryptor = RSA.importKey(publickey, passphrase="f00bar")
#     retval=None
#     file = open("/Users/yuyongpeng/git/hard-chain/cport/oracle/contracts//cryptThingy.txt", "rb")
#     retval = decryptor.decrypt(file.read())
#     file.close()
#     return retval

# if __name__ == "__main__":
#     decryptedThingy=decrypt()
#     print "Decrypted: %s" % decryptedThingy


def test():
    import base64

    from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
    from Crypto.PublicKey import RSA

    message = b"this is test"

    rsa_private_key = b'-----BEGIN RSA PRIVATE KEY-----\nMIIEowIBAAKCAQEAz4C8rDjl6NAAvdUxLl7p8eA+xxc4Sa20puq2gFPk+8B5W30o+8xAaktLoji9wEsf7WTEllxZcIXw7d4K9b1M34ZDQT1iR4ZreB3GkxEXDeQIV3f/W7C3NL1bF5ePn7DRGZYKTEry8pp/lK/1ESP9Z+giaN29xm+jsmvNAOJlLjndn8nYcmA1sdDzuX8JSlZOiijAwyKUkwrBd+FqlhbAfF9ebcEgQcRISZsBMpcsZ66HQ0nvEnhmLzkfXxLUu2w7ccwLc/W7a4V1SNFEMZifRyiI7IhYYpRFw7egQN2RgzVWW/O6HKzk2Mxaz/6Y+HEehuDfRDn5MjMCkhnfbpRKKwIDAQABAoIBAD7xpkn32IYO0qrDCPAwCnc8ts5d1M1EzA/2EvZKfKOBd06xYMVA6ClWPdMvsx+pqj2w25Fg4iCoALWnLVvt2GvVpoEbxbw8ok0Fez8RaBTmn0EpPWTq7igpaIB3kTG3yTYhtW3/LkM5aooH9icE69GhMQ7udKrFIrr0yHMEOf0TNKBUNlwwHWJrpUuodXxIbERonkNdri1AnsWsnh4ePkKFlgaIdOZ24W4kHjfQAQkz3bPTjRzU5Q0LpkCpuYxk+TeOvR5tNPHXtzTWYek7JbCbnIKr1SiJx+ncwWM7QEnuvbzCU2VeTwb6Rq94WIg3RpfqSKJPr5w5e/df4jmi58ECgYEA74ty4UiChdp27/xvy2sJMilgkcL5Z0A+McPrjM82CqfPvr1+ZMBkTwuQGxFWlNtdE7DLpneVCWmzAexUJ02dFndSJMqsN81335VYfLc94qk5HXwmc9UBK7+qL2Ugyz76KoRGogqkKYw0ZR2kdpcNoXjQfeVgqrumt+jnZChpAl8CgYEA3cHPuDbdqqhcslz4+Zz8B0zWKN8Kj5CgG9ReEmJGpE6w5D7bds5iPcmp5xwFctuumAtroA/+bpGX/2oXc1nXucf/dI3AWoaBQrZ9iL6xfe7+ZPiRYuZjz3ygZ+Vcgp6xixbTT86jaIHwsD1wYtzOiiYOUBt4jDk89xookjpBg7UCgYEAtCriQOdUpHBoPKBVRV5AEGZmp7tJ8oqzPKLrK4E+WE1XR1MnGYhK6dj8jh9AS72iCAlVYuNWSToi1TN3KAiMOjWMpvd5CI+VtaiSYVtBRJWay75w/XYb51fFHNinDbdUWV4b8gym5Ej7r4HYDQoXyncf+VDooAF7p5+ZSg/Ky2kCgYAlVsv+Dab3ZE/vbH4zsX4yUHrC+QNTNxvuc6y/VbLlWaapV/gmIgwisUEde/di5qCYU1v04JyLy5IXXKrELn2Hd6iI3JFl7L35GLc8fjdup/5HzB7W00o9FpP/ynvwNn5YmChOiNG0+CQp2L6CtwFD+7JvRKDgo4ajaNEqBgsf/QKBgE6kbfW6FSpFdOs+2qpIXTEWnTqUC3WD1h1qB8G8J3Tn/o/bSm8C5lHmjmg9vRJtHisaoLyc2ikH7UZwPGU32340s22igkDZml730CKrYtnnBZAG+tIiERtI4Zj432X7qzBYgTgwO7wRAvds+WDCwwV5FMGNCzbPEXos8HmS0812\n-----END RSA PRIVATE KEY-----\n'

    rsa_public_key = b'-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAz4C8rDjl6NAAvdUxLl7p8eA+xxc4Sa20puq2gFPk+8B5W30o+8xAaktLoji9wEsf7WTEllxZcIXw7d4K9b1M34ZDQT1iR4ZreB3GkxEXDeQIV3f/W7C3NL1bF5ePn7DRGZYKTEry8pp/lK/1ESP9Z+giaN29xm+jsmvNAOJlLjndn8nYcmA1sdDzuX8JSlZOiijAwyKUkwrBd+FqlhbAfF9ebcEgQcRISZsBMpcsZ66HQ0nvEnhmLzkfXxLUu2w7ccwLc/W7a4V1SNFEMZifRyiI7IhYYpRFw7egQN2RgzVWW/O6HKzk2Mxaz/6Y+HEehuDfRDn5MjMCkhnfbpRKKwIDAQAB\n-----END PUBLIC KEY-----\n'

    s = b'Ho8r8K07ZuhY8DAG5ACmCRx+8LPe7YJiZ3ZefgnCohjXlTaBxbIs5KbU5jdVvQF3qesImTRTULa7rka5Ms7ymey6OGceKc3wT1h6xp542qVzlhBK87bHE++IIJls73uVWii99c6x+ow2L5rrZtsgOZFn4CGDsDtlXK71zsvZfZxWWiQYy67G0AayzY2c621cZ55VWY6Z4CVgd5i6W1tnsLows3c8gMKBsozFLovGle5lp+sqL39fNdpwbEas3qfNbqxP0ujBOGM2JSajGlQcLrJPlPnQGoT3DIydgHXbmAvq3DkW5siQJWiA+UwG57cGSkUDneY+oowUGgYtPJrl0g=='
    rsakey = RSA.importKey(rsa_public_key)
    cipher = Cipher_pkcs1_v1_5.new(rsakey)
    cipher_text = base64.b64encode(cipher.encrypt(message))
    print(cipher_text)

    from oracle.crypto import encrypt2, decrypt2

    p = encrypt2(message, rsa_public_key)
    print(p)
    print(decrypt2(s, rsa_private_key))

    # rsakey = RSA.importKey(rsa_private_key)
    # cipher = Cipher_pkcs1_v1_5.new(rsakey)
    # text = cipher.decrypt(base64.b64decode(cipher_text), None)
    # print(text.decode('utf8'))


import rsa
from rsa import PublicKey


# from oracle.crypoor import encrypt,decrypt


def test_cryptography():
    from oracle.crypto import generate_private_key, generate_public_key, decrypt_rsa, encrypt_rsa
    (private_key_obj, private_key_str) = generate_private_key()
    (public_key_obj, public_key_str) = generate_public_key(private_key_obj)
    print(private_key_str)
    print(public_key_str)
    message = "hello world"
    sec = encrypt_rsa(message, public_key_obj)
    print(sec)
    import base64
    print(base64.b64decode(sec))
    txt = decrypt_rsa(sec, private_key_obj)
    print(txt)
    # print(repr(base64.b64encode(sec)))


def test_ecies():
    from oracle.crypto import encrypt_ecies, check_ecies
    private_Key = '0x6a6d5f34772ef3ba498c5256aa8470d2401e82a5921cbef58ec0c52e4336b925'
    public_Key = '0x044af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8916ac219bb70e64777930b1e85d0a550e1e61ca94fa902dee419849a9a0efe8a'
    original_text = 'abcedf'
    iv = '0x0c174c174e0c5a1643177d2e4ed3de8a'
    ciphertext = '0xb7db50ee715d37c0b73c1f0dd022990a'
    ephemPublicKey = '0x04be7916ed4da0f8b5bf44a211b4a41ce0998664eb5350bf16337c61160706a1da7892fef570651084f95fcc4d8ff2133f854a7247af2235b90c20a805ca4cb35a'
    iv = '0x0c174c174e0c5a1643177d2e4ed3de8a'
    mac = '0x78c35d9d22fb1050458c7e41e788006a91b7ef59ddd985c013fc08049b8935ce'
    print(encrypt_ecies(public_Key, original_text, iv))
    # tx = check_ecies(private_key=private_key, ephemPublicKey=ephemPublicKey, ciphertext=ciphertext,verified_text='abcdef')
    # print(tx)


def test_iv():
    from oracle.crypto import get_iv
    import datetime
    now = datetime.datetime.now()
    get_iv('1357723206')


def test_sign():
    from eth_keys import keys as k
    pk = bytes.fromhex('6a6d5f34772ef3ba498c5256aa8470d2401e82a5921cbef58ec0c52e4336b925')
    pk2 = k.PrivateKey(pk)
    pk2.public_key

    message = b'{"@context":"https://w3id.org/did/v1","id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a#soldier","publicKey":[{"id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a##proof","type":"EciesVerificationKey","controller":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a","publicKeyHex":"0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297"}],"authentication":[{"id":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2#proof","type":"EciesVerificationKey","controller":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2","publicKeyHex":"0x024af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8"}],"proof":"6617fd92f9f8b9b930ff051f6c02ece8d27cf0b2912361093ce38e521bc3c4846885eec37fefdbb06ebc699fcaeb4136","created":1554184638038}\n04a469fb703b394646a995e6681771967beea3f1ca8fe7b370555e4c5a6dca5b071ea5d9bbec76e3b1058b54894ddff9f9d346934f74a511d17bf56e6cb892fa01'
    army_privatekey = '0x6a6d5f34772ef3ba498c5256aa8470d2401e82a5921cbef58ec0c52e4336b925'
    army_publickey = '0x044af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8916ac219bb70e64777930b1e85d0a550e1e61ca94fa902dee419849a9a0efe8a'
    from pycontractsdk import keys
    sign = keys.ecdsa_sign(message, army_privatekey)
    print(sign)

    tf = keys.ecdsa_verify(message.decode(), sign, army_publickey)
    print(tf)

if __name__ == "__main__":
    # test_cryptography()
    # test_ecies()
    # test_iv()
    test_sign()




#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: rabbitmq 的消费者程序
#  

"""

from abc import ABCMeta, abstractmethod
import pika
import json
import re
import toolz
from oracle.setting import VAR
from oracle.database import dbs
from oracle.logger import logger
from oracle.queues import Rabbitmq, Kafka
from datetime import datetime
import functools

class Cusomer(object):
    __metaclass__ = ABCMeta

    def __init__(self, *args, **kwargs):
        self._queue_name = kwargs.pop('queue_name')
        self._queue_user = kwargs.pop('queue_user')
        self._queue_password = kwargs.pop('queue_password')
        self._queue_ip = kwargs.pop('queue_ip')
        self._queue_port = kwargs.pop('queue_port')
        self._queue_vhost = kwargs.pop('queue_vhost')
        self._contract = kwargs.pop('contracrt')

    @abstractmethod
    def cusomer(self):
        pass

    def parse_arg(self, function_args):
        """
        解析json，获得上链的参数
        :param function_args:  {} 对象
        :return:
        """
        args = []
        for arg in function_args:
            name = arg['name']
            value = arg['value']
            arg_type = arg['type']
            source_type = arg.get('source_type', 'string')
            if re.match('bytes.*', arg_type):
                if source_type == 'string':
                    value = value.encode()
                if source_type == 'hex':
                    value = bytes.fromhex(value)
                args.append(value)
            if re.match('.*int.*', arg_type):
                value = int(value)
                args.append(value)
            if re.match('address', arg_type):
                args.append(value)
            if re.match('string', arg_type):
                args.append(value)
            if re.match('bool', arg_type):
                args.append(value)
        return args


    def upload_chain(self, body, queue_flow, wait=False):
        """
        上链操作
        :param body: 消息体  json串
        :return:
        """
        try:
            body_item = json.loads(body)
            queue_flow_id = queue_flow.id;
            body_item.update({'queue_flow': queue_flow_id})
            type = body_item['type']
            retry = body_item.get('retry', 0)  # 第几次重试
            upload_time = body_item.get('upload_time', datetime.fromtimestamp(datetime.now().timestamp()).strftime('%Y-%m-%d %H:%M:%S'))
            if type == 'call_function':
                function_name = body_item['function']['name']
                function_args = body_item['function']['args']
                # 解析args参数，获得调用协约需要的参数列表
                args = self.parse_arg(function_args)
                # 上链
                tf, tx_hash = self._contract.call_function(function_name, True, *args)
                # 更新 receipt hash
                queue_flow.update(receipt=tx_hash).where(queue_flow.id == id).execute()

                # 发送到监听队列，监听是否上链
                body_item.update({'receipt': tx_hash})
                body_item.update({'upload_time': upload_time})
                body_item.update({'retry': int(retry)+1})
                rabbitmq = Rabbitmq(queue_ip=VAR['QUEUE_IP'], queue_port=VAR['QUEUE_PORT'], queue_user=VAR['QUEUE_USER'],
                                    queue_password=VAR['QUEUE_PASSWORD'], queue_name=VAR['QUEUE_NAME_VALIDATION'],
                                    queue_vhost=VAR['QUEUE_VHOST'])
                rabbitmq.send_msg(json.dumps(body_item))

            if type == 'upload_chain':
                data = body_item['data']['msg']
                # TODO: 直接对数据进行上链操作

        except json.JSONDecodeError as j:
            logger.exception('json 解析错误 ')
            logger.error(j)

    def send_notice(self, success, body):
        """
        发送通知
        :param success: 是否上链成功
        :param body:    需要处理的对象，obj
        :return:
        """
        body_item = body
        body_item.update({'success': str(success).lower()})
        queue_msg = json.dumps(body_item)
        # queue
        if 'send_queue' in body_item:
            queue_type = body_item['send_queue']['queue_type']
            if queue_type == "rabbitmq":
                queue_ip = body_item.get('queue_ip', VAR['QUEUE_IP'])
                queue_port = body_item.get('queue_port', VAR['QUEUE_PORT'])
                queue_user = body_item.get('queue_user', VAR['QUEUE_USER'])
                queue_password = body_item.get('queue_password', VAR['QUEUE_PASSWORD'])
                queue_name = toolz.get_in(['send_queue', 'queue_msg', 'queue_name'], body_item, VAR['QUEUE_NAME_NOTICE'])
                queue_vhost = body_item.get('queue_vhost', VAR['QUEUE_VHOST'])
                rabbitmq = Rabbitmq(queue_ip=queue_ip, queue_port=queue_port, queue_user=queue_user,
                                    queue_password=queue_password, queue_name=queue_name, queue_vhost=queue_vhost)
                rabbitmq.send_msg(queue_msg)
            if queue_type == "kafka":
                # TODO: 还没有测试 kafka部分
                queue_ip = body_item.get('queue_ip', VAR['KAFKA_IP'])
                queue_port = body_item.get('queue_port', VAR['KAFKA_PORT'])
                queue_topic = body_item.get('queue_topic', VAR['QUEUE_TOPIC'])
                kafka = Kafka(queue_ip=queue_ip, queue_port=queue_port, queue_topic=queue_topic)
                kafka.send_msg(queue_msg)

        # database
        if 'send_database' in body_item:
            database = body_item['send_database']
            if 'mysql' in database:
                dbs.update_mysql(success=success, **database['mysql'])
            if 'postgresql' in database:
                # TODO: 还没有实现 postgresql部分
                pass

        # mail
        if 'send_email' in body_item:
            email = body_item['send_email'] # list
            from oracle import mail
            mail_subject = "上链成功" if success else "上链失败"
            mail.send_email(mail_subject=mail_subject,
                            mail_host=VAR['MAIL_SMTP'],
                            mail_pass=VAR['MAIL_PASSWORD'],
                            mail_receiver=email,
                            mail_body=queue_msg,
                            mail_sender=VAR['MAIL_USER'])


class CallContract(Cusomer):
    """
    用于上链操作
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def callback(self, ch, method, properties, body):
        """
        队列处理的业务逻辑
        :param ch:          channel
        :param method:
        :param properties:
        :param body:        需要处理的消息体 json
        :return:
        """
        logger.info(body)
        try:
            body_item = json.loads(body)
            # TODO: 把body信息存入数据库中，以备后查
            queue_flow = dbs.get_queue_flow(body)
            retry = body_item.get('retry', 0)           # 第几次重试
            max_retry = body_item.get('max_retry', 3)   # 最大的重试次数
            if retry > max_retry:
                logger.error('************************************************************')
                logger.error('** 重试次数大于{}, 不在进行操作，请查找原因'.format(max_retry))
                logger.error(body)
                logger.error('************************************************************')
                return
            # 上链操作
            self.upload_chain(body, queue_flow)
            ch.basic_ack(delivery_tag=method.delivery_tag)
        except json.JSONDecodeError as j:
            ch.basic_ack(delivery_tag=method.delivery_tag)
            logger.exception('json 解析错误 ')
            logger.error(j)
        except Exception as e:
            ch.basic_ack(delivery_tag=method.delivery_tag)
            logger.error('************************************************************')
            logger.error('** JSON 数据处理遇到无法挽回的错误 ')
            logger.error(e)
            logger.error(body)
            logger.error('************************************************************')

    # @retry(pika.exceptions.AMQPConnectionError, delay=5, jitter=(1, 3))
    def cusomer(self):
        # 连接rabbitmq
        credentials = pika.PlainCredentials(self._queue_user, self._queue_password)
        connection = pika.BlockingConnection(
            pika.ConnectionParameters(self._queue_ip, self._queue_port, self._queue_vhost, credentials))
        channel = connection.channel()
        # channel.queue_delete(queue='cport_call_contract    # 专门用于调用智能协约')
        # exit(1)
        channel.queue_declare(queue=self._queue_name,   # 队列名称
                              passive=False,            # 仅检查队列是否存在
                              durable=False,            # true: 启动不允许修改queue_declare参数，如果有修改，就会报错
                              exclusive=False,          # 只允许通过当前连接访问
                              auto_delete=False,        # 断开连接后自动删除这个队列
                              )

        # 公平调度，使得rabbitmq不会在同一时间给工作者分配多个任务，即只有工作者完成任务之后，才会再次接收到任务。
        channel.basic_qos(prefetch_count=1)
        # 消费
        try:
            channel.basic_consume(queue=self._queue_name,
                                  on_message_callback=self.callback,
                                  auto_ack=False,                # 自动确认模式
                                  exclusive=False,               # 独占模式，不允许有其他的消费者
                                  consumer_tag='call_contract',  # 指定消费者标签
                                  )
        except Exception as ex:
            logger.error(ex)

        logger.info('[*] Waiting for messages. To exit press CTRL+C')
        channel.start_consuming()


class ValidationContract(Cusomer):
    """
    用于验证上链是否成功
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def callback(self, ch, method, properties, body):
        """
        监测是否上链完成
        :param ch:          pika.Channel
        :param method:      pika.spec.Basic.Deliver
        :param properties:  pika.spec.BasicProperties
        :param body:        bytes
        :return:
        """
        logger.info(body)
        try:
            body_item = json.loads(body)
            # 把body信息存入数据库中，以备后查
            queue_flow = dbs.get_queue_flow(body)

            retry = body_item.get('retry', 0)  # 第几次重试
            max_retry = body_item.get('max_retry', 3)  # 最大的重试次数

            upload_time = datetime.strptime(body_item['upload_time'], '%Y-%m-%d %H:%M:%S')
            timeout = int(body_item['upload_timeout'])
            now = int(datetime.now().timestamp())
            send_msg = functools.partial(Rabbitmq, queue_ip=VAR['QUEUE_IP'], queue_port=VAR['QUEUE_PORT'],
                                         queue_user=VAR['QUEUE_USER'], queue_password=VAR['QUEUE_PASSWORD'],
                                         queue_vhost=VAR['QUEUE_VHOST'])
            # 查看是否存在 receipt 的 tx_hash
            if 'receipt' not in body_item:
                logger.error('************************************************************')
                logger.error('** JSON 数据中 不存在 receipt hash，没法处理，发回上链队列重新上链')
                logger.error(body)
                logger.error('************************************************************')
                ch.basic_ack(delivery_tag=method.delivery_tag)

            # 查看数据是否上链成功
            tf, receipt = self._contract.is_tx_sucess(body_item['receipt'])
            if tf is True: # 上链 完成 发送通知
                body_item.update({'success': True})
                self.send_notice(tf, body_item)
            else: # 上链 未完成 重新进入查询队列
                if (now - int(upload_time.timestamp())) > timeout:  # 超时 发送回上链队列，重新进行上链操作
                    if retry >= max_retry:
                        body_item.update({'success': False})
                        self.send_notice(tf, body_item)
                        ch.basic_ack(delivery_tag=method.delivery_tag)
                        return
                    else:
                        rabbitmq = send_msg(queue_name=VAR['QUEUE_NAME'])
                        rabbitmq.send_msg(json.dumps(body_item))
                else:
                    rabbitmq = send_msg(queue_name=VAR['QUEUE_NAME_VALIDATION'])
                    rabbitmq.send_msg(json.dumps(body_item))

                rabbitmq = send_msg(queue_name=VAR['QUEUE_NAME_VALIDATION'])
                rabbitmq.send_msg(json.dumps(body_item))

            ch.basic_ack(delivery_tag=method.delivery_tag)
        except json.JSONDecodeError as j:
            ch.basic_ack(delivery_tag=method.delivery_tag)
            logger.exception('json 解析错误 ')
            logger.error(j)
        except Exception as e:
            ch.basic_ack(delivery_tag=method.delivery_tag)
            logger.error('************************************************************')
            logger.error('** JSON 数据处理遇到无法挽回的错误 ')
            logger.error(e)
            logger.error(body)
            logger.error('************************************************************')

    # @retry(pika.exceptions.AMQPConnectionError, delay=5, jitter=(1, 3))
    def cusomer(self):
        # 连接rabbitmq
        credentials = pika.PlainCredentials(self._queue_user, self._queue_password)
        connection = pika.BlockingConnection(
            pika.ConnectionParameters(self._queue_ip, self._queue_port, self._queue_vhost, credentials))
        channel = connection.channel()
        # self._queue_name = 'cport_notice_contract'
        channel.queue_declare(queue=self._queue_name,   # 队列名称
                              passive=False,            # 仅检查队列是否存在
                              durable=False,            # 在代理重新启动后仍然是存活的
                              exclusive=False,          # 只允许通过当前连接访问
                              auto_delete=False)        # 断开连接后自动删除这个队列

        # 公平调度，使得rabbitmq不会在同一时间给工作者分配多个任务，即只有工作者完成任务之后，才会再次接收到任务。
        channel.basic_qos(prefetch_count=1)
        # 消费
        try:
            channel.basic_consume(queue=self._queue_name,
                                  on_message_callback=self.callback,
                                  auto_ack=False,    # 自动确认模式
                                  exclusive=False,   # 独占模式，不允许有其他的消费者
                                  consumer_tag='ValidationContract',    # 指定消费者标签
                                  )
        except Exception as ex:
            logger.error(ex)

        logger.info('[*] Waiting for messages. To exit press CTRL+C')
        channel.start_consuming()


import multiprocessing
from pycontractsdk.contracts import Contract
class ValidationChainWorker(multiprocessing.Process):
    def __init__(self, intval):
        multiprocessing.Process.__init__(self)
        self.intval = intval

    def run (self):
        # 实例化 Contract
        provider = VAR['ETH_PROVIDER']
        contract_address = VAR['CONTRACT_ADDRESS']
        abi = VAR['CONTRACT_ABI']
        operator_private_key = VAR['OPERATOR_PRIVATEKEY']
        delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
        gas = VAR['GAS']
        gas_prise = VAR['GAS_PRISE']
        concart = Contract(provider=provider, timeout=60, contract_address=contract_address, abi=abi,
                           private_key=operator_private_key,
                           gas=gas,
                           gas_prise=gas_prise
                           )
        # 启动 消费者程序
        cus = ValidationContract(
            contracrt=concart,
            queue_name=VAR['QUEUE_NAME_VALIDATION'],
            queue_ip=VAR['QUEUE_IP'],
            queue_port=VAR['QUEUE_PORT'],
            queue_user=VAR['QUEUE_USER'],
            queue_password=VAR['QUEUE_PASSWORD'],
            queue_vhost=VAR['QUEUE_VHOST'],
        )
        cus.cusomer()



#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""

from wechatpy import WeChatClient
from redis import Redis
from wechatpy.session.redisstorage import RedisStorage
from wechatpy.exceptions import WeChatClientException

app_id = 'wx9af3e63ca6cc792c'
secret = 'f4d4661134a9694f9ebf1a60c8742007'
user_id = 'oGdq9s0W1DYA1TbMIH2tXPCiz3aA'
client = WeChatClient(app_id, secret)
print(client.access_token)
user = client.user.get(user_id)
# menu = client.menu.get()
redis_client = Redis.from_url('redis://127.0.0.1:6379/0')
session_interface = RedisStorage(
    redis_client,
    prefix="wechatpy"
)

wechat_client = WeChatClient(
    app_id,
    secret,
    session=session_interface
)



client.message.send_text(user_id, 'content')
try:
    res = wechat_client.user.get('oGdq9s0W1DYA1TbMIH2tXPCiz3aa')
    print(res)
except WeChatClientException as e:
    print(e)
    print(e.errcode)
# 以此类推，参见下面的 API 说明
# client.media.xxx()
# client.group.xxx()

































#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description:
#  

"""

BROKER_URL = "redis://127.0.0.1:6379/1"
CELERY_RESULT_BACKEND = "redis://127.0.0.1:6379/2"




#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 微信相关的函数
#  

"""

from oracle.setting import VAR

from wechatpy import WeChatClient
from redis import Redis
from wechatpy.session.redisstorage import RedisStorage
from wechatpy.exceptions import WeChatClientException


def get_client():
    redis_client = Redis.from_url('redis://'.join([VAR['REDIS_IP'], ':', VAR['REDIS_PORT'], '/', VAR['REDIS_DATABASE']]))
    session_interface = RedisStorage(
        redis_client,
        prefix="wechatpy"
    )
    wechat_client = WeChatClient(
        VAR['WECHAT_APPID'],
        VAR['WECHAT_SECRET'],
        session=session_interface
    )
    return wechat_client


def send_msg(weixin_openid, msg):
    """
    给订阅了公众号的用户发送信息
    :param weixin_openid: 已经订阅了公众号的用户  OpenID
    :param msg: 要发送的信息
    :return:
    """
    wechat_client = get_client()
    # print(wechat_client.access_token)
    wechat_client.message.send_text(weixin_openid, msg)


def is_user(user_id):
    """
    查询这个用户是否订阅的公众号
    :param user_id: 已经订阅了公众号的用户  OpenID
    :return: 所有分组列表或用户所在分组 ID
    """
    wechat_client = get_client()
    try:
        wechat_client.user.get(user_id)
    except WeChatClientException as e:
        if '40003' == e.errcode:
            return False
    else:
        return True

















#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-22 18:59:53
LastEditors:  
LastEditTime: 2019-04-22 18:59:53
Description:  
"""





#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-17 17:25:06
LastEditors:  
LastEditTime: 2019-04-17 17:25:06
Description:  
"""

import json
from oracle.plugins import Processor
from oracle.database import dbs
from oracle.database import status
from oracle.dids import Did
from oracle.setting import VAR
from oracle.logger import logger
from oracle.database.modles import Army, Organization
from pycontractsdk import keys
from oracle import wechat
from pycontractsdk.contracts import Cport

@Processor.plugin_register('DIDAttributeChange')
class DIDAttributeChangeEvent(object):
    """
    处理 event = DIDAttributeChange 的数据
    """
    def __init__(self):
        pass

    def process(self, entry):
        """
        处理 event=DIDAttributeChange 的函数
        :param entry: 获得的event数据
        :return:
        """
        args = entry['args']
        name = args['name'].strip(b'\x00')
        name = name.decode('utf8')
        # 所有已经注册的插件 都运行
        processor = Processor()
        processor.process_identity(event='DIDAttributeChange' ,identity=name, entry=entry)


@Processor.plugin_register('DIDAttributeConfirmed')
class DIDAttributeConfirmedEvent(object):
    """
    处理 event = DIDAttributeConfirmed 的数据
    """
    def __init__(self):
        pass

    # def did_attribute_confirmed(self, cport, entry, army_private_key, operator_private_key, delegate_private_key):
    def did_attribute_confirmed(self, cport, entry, *args, **kwargs):
        """
        处理 event=DIDAttributeConfirmed 的函数
        验证 机构 上链的数据
        确认军人的身份之后，进行上链操作，告诉大家他是一个军人

        获得的event数据，比 DIDAttributeChange 多了一个签名
        AttributeDict(
        {'args': AttributeDict(
          {
            'identity': '0xB2be7c8722935DB16921D64Ad0dDB5D47A52482a',
            'name': b'\xe5\xa3\xab\xe5\x85\xb5\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
            'value': b'{"@context":"https://w3id.org/did/v1","id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a#soldier",
                        "publicKey":[{"id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a##proof",
                                      "type":"EciesVerificationKey",
                                      "controller":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a",
                                      "publicKeyHex":"0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297"}],
                        "authentication":[{"id":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2#proof",
                                            "type":"EciesVerificationKey",
                                            "controller":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2",
                                            "publicKeyHex":"0x024af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8"}],
                        "proof":"6617fd92f9f8b9b930ff051f6c02ece8d27cf0b2912361093ce38e521bc3c4846885eec37fefdbb06ebc699fcaeb4136",
                        "created":1554184638038}
                        \n
                        04a469fb703b394646a995e6681771967beea3f1ca8fe7b370555e4c5a6dca5b071ea5d9bbec76e3b1058b54894ddff9f9d346934f74a511d17bf56e6cb892fa01
                        \n
                        04a469fb703b394646a995e6681771967beea3f1ca8fe7b370555e4c5a6dca5b071ea5d9bbec76e3b1058b54894ddff9f9d346934f74a511d17bf56e6cb892fa01
                        '
          }
        ),
        'event': 'DIDAttributeConfirmed',
        'logIndex': 0,
        'transactionIndex': 0,
        'transactionHash': HexBytes('0xe453910d44ad566093d33593f6411fd73126564617bd1529fedce6fe42be0fa7'),
        'address': '0x5c12AED6613AaD77eC2e84e5cedd9b6Ff29A6e6A',
        'blockHash': HexBytes('0x0c602e8ad51cb5a49ab4952181a252f0bf6f55c9f3fc0cbaa043a9f2b8e5814e'),
        'blockNumber': 293710}
        )

        :param cport: pycontractsdk中的针对 cport 协约的处理对象
        :param entry: 以太坊获得的event数据
                    event DIDAttributeConfirmed(address indexed identity, bytes32 name, bytes value) 函数的参数
        :return:
        """
        # XXX: 需要确认是否会抛异常
        army_private_key = kwargs['army_private_key']           # 军队账号的私钥
        operator_private_key = kwargs['operator_private_key']   # 管理者的私钥
        delegate_private_key = kwargs['delegate_private_key']   # 代理上链者的私钥

        # 数据存到数据库中
        event = dbs.save_organization(entry)
        id = event.id
        args = entry['args']
        identity = args['identity']                         # identity
        name = args['name'].strip(b'\x00')                  # 身份信息描述，比如"士兵"
        value = args['value']                               # did和签名信息
        first_sign_text_bytes = value.split(b'\n')[-2]      # 第一个 签名信息
        second_sign_text_bytes = value.split(b'\n')[-1]     # 第二个 签名信息
        did_json_bytes = b''.join(value.split(b'\n')[:-2])  # did 信息
        did_json_obj = json.loads(did_json_bytes.decode())  # did json对象

        # 验证 did 的签名是否正确 first_sign
        # 第一个签名是用户发起的。必须确认。did['publicKey'][0]['publicKeyHex']
        if not Did.verify_did(did_json_bytes.decode(), first_sign_text_bytes.decode()):
            logger.error('*******************************************************')
            logger.error('* First signature verification error !')
            logger.error('* did: {}'.format(did_json_bytes.decode()))
            logger.error('* sign_text: {}'.format(first_sign_text_bytes.decode()))
            logger.error('*******************************************************')
            # 需要修改数据库的状态为 "验证第一个签名失败"
            Organization.update(status=status.ORG_FIRST_SIGN_VERI_FAILED).where(Organization.id == id).execute()
            return
        # 需要修改数据库的状态为 "验证第一个签名成功"
        Organization.update(status=status.ORG_FIRST_SIGN_VERI_SUCCESS).where(Organization.id == id).execute()

        # 验证 (did + first_sign) 的签名是否正确 second_sign
        # 第二个签名是军队发起的。必须确认。did['authentication'][0]['publicKeyHex']
        message = b'\n'.join(value.split(b'\n')[:-1]).decode()
        # army_public_key = '0x044af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8916ac219bb70e64777930b1e85d0a550e1e61ca94fa902dee419849a9a0efe8a'
        tf = keys.ecdsa_verify(message, signature=second_sign_text_bytes.decode(),
                               public_key=did_json_obj['authentication'][0]['publicKeyHex'])
                                # public_key=army_public_key)
        if tf is False:
            logger.error('*******************************************************')
            logger.error('* second signature verification error !')
            logger.error('* did+first_sign: {}'.format(message))
            logger.error('* sign_text: {}'.format(second_sign_text_bytes.decode()))
            logger.error('*******************************************************')
            # 需要修改数据库的状态为 "验证第二个签名失败"
            Organization.update(status=status.ORG_SECOND_SIGN_VERI_FAILED).where(Organization.id == id).execute()
            return
        # 需要修改数据库的状态为 "验证第二个签名成功"
        Organization.update(status=status.ORG_SECOND_SIGN_VERI_SUCCESS).where(Organization.id == id).execute()

        # 保存验证通过的数据到最终表中，以最后一次的数据为准
        dbs.save_organization_did(entry)

        # 给用户发送微信
        weixin_openid = dbs.get_openid(identity)
        if weixin_openid is None:
            Organization.update(status=status.ORG_SEND_WECHAT_FAILED).where(Organization.id == id).execute()
        else:
            wechat.send_msg(weixin_openid, VAR['WEIXIN_MSG_TMP'].format(identity, name))
            Organization.update(status=status.ORG_SEND_WECHAT_SUCCESS).where(Organization.id == id).execute()


    def process(self, entry):
        """
        处理 event=DIDAttributeConfirmed 的函数
        :param entry: 获得的event数据
        :return:
        """
        provider = VAR['ETH_PROVIDER']
        contract_address = VAR['CONTRACT_ADDRESS']
        abi = VAR['CONTRACT_ABI']
        army_private_key = VAR['ARMY_PRIVATEKEY']
        operator_private_key = VAR['OPERATOR_PRIVATEKEY']
        delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
        cport = Cport(provider=provider, timeout=60, contract_address=contract_address, abi=abi,
                      private_key=operator_private_key
                      # delegate_private_key=delegate_private_key
                      )
        self.did_attribute_confirmed(cport=cport,
                                  army_private_key=army_private_key,
                                  private_key=operator_private_key,
                                  # operator_private_key=operator_private_key,
                                  # delegate_private_key=delegate_private_key,
                                  entry=entry
                                  )



#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-08 17:40:12
LastEditors:  
LastEditTime: 2019-04-08 17:40:12
Description:  
"""

import pika
from oracle import setting
from oracle.setting import VAR

def send_msg(file):
    """ 发送消息 """
    # 初始化配置信息
    setting.read_config2(file)
    # connection = pika.BlockingConnection(pika.ConnectionParameters(host='localhost'))
    credentials = pika.PlainCredentials('hard', 'chain')
    connection = pika.BlockingConnection(pika.ConnectionParameters(VAR['QUEUE_IP'], VAR['QUEUE_PORT'], VAR['QUEUE_VHOST'], credentials))
    channel = connection.channel()
    body = '''  
    {
  "type":"call_function",
  "retry": 0,                   
  "max_retry": 3,              
  "upload_timeout": 300,    
  "send_database": {
    "mysql": {
      "schema": "cport",
      "table": "person_dc",   
      "id": "123456",    
      "id_column": "id", 
      "column": "status", 
      "success_tag": "1", 
      "failed_tag": "2" 
    }
  },
  "send_email": ["yuyongpeng@hotmail.com"],
  "send_queue":{
    "queue_type": "rabbitmq",
    "queue_msg": {
      "queue_name": "cport_notice_contract"
    }
  },
  "function":{
    "name": "setAttributeSigned", 
    "args": [       
      {
        "name": "identity",  
        "value": "0xB2be7c8722935DB16921D64Ad0dDB5D47A52482a",     
        "type": "address"  
      },
      {
        "name": "sigV",
        "value": "00",
        "source_type": "hex",
        "type": "uint8"
      },
      {
        "name": "sigR",
        "value": "1bac8f2eeaf962ccf9ace9845c40f553453fb0079d80173dfcc7daf051240917",
        "source_type": "hex",
        "type": "bytes32"
      },
      {
        "name": "sigS",
        "value": "34888f4ba7b8591025fddafbba5b9e3c229a49073daeaa203d4b0260aa09b775",
        "source_type": "hex",
        "type": "bytes32"
      },
      {
        "name": "name",
        "value": "士兵",
        "type": "bytes32"
      },
      {
        "name": "value",
        "value": "{\\"@context\\":\\"https://w3id.org/did/v1\\",\\"id\\":\\"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a#soldier\\",\\"publicKey\\":[{\\"id\\":\\"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a#soldier\\",\\"type\\":\\"EciesVerificationKey\\",\\"controller\\":\\"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a\\",\\"publicKeyHex\\":\\"0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297\\"}],\\"authentication\\":[{\\"id\\":\\"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2#soldier\\",\\"type\\":\\"EciesVerificationKey\\",\\"controller\\":\\"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2\\",\\"publicKeyHex\\":\\"0x044af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8916ac219bb70e64777930b1e85d0a550e1e61ca94fa902dee419849a9a0efe8a\\"}],\\"proof\\":\\"5a20a8ab74ba21186197c5315369dea7eb50a9700a0d88c49c360f9e3e926880b7e8f793acd91800c33c2d64ae9c846c\\",\\"created\\":1554808209789}\\nb4f2d5447a602dca239d8ab528e1cb1b0ddc409d0941700fcffed5e6bd427c2755658c7e7790a94f69417ce43c5b31c9d82e9b2900d77afcec6d4268d4e446d300",
        "type": "bytes"
      }
    ]
  }
}
    '''

    import json
    # b = json.loads(body)
    # channel.queue_delete(queue='cport_notice_contract')
    # exit(0)
    channel.queue_declare(queue=VAR['QUEUE_NAME'], durable=False)

    channel.basic_publish(exchange='',
                          routing_key=VAR['QUEUE_NAME'],
                          body=body)
    print(body)
    connection.close()


if __name__ == '__main__':
    send_msg("/Users/yuyongpeng/git/hard-chain/cport/oracle/oracle.conf")



#!/usr/bin/env python
# -*- coding=UTF-8 -*-

from oracle.business import BaseEvent, ArmyEvent, import_event, OrganizationEvent
from oracle import setting
import click
from oracle import cusomer
from oracle.setting import VAR
from pycontractsdk.contracts import Contract
from oracle.database import dbs
from oracle.plugins import Processor
# from oracle.plugins.solider import Solider
import importlib
import os
import re
from oracle.utils import import_plugins




@click.command()
@click.option('-f', "--file", default='/etc/oracle_army.conf', help='指定配置文件的路径（默认: /etc/oracle_army.conf）')
def army(file):
    # TODO: 军队 运行的验证程序"
    # 初始化配置信息
    setting.read_config2(file)
    # 导入`event` 插件
    import_plugins()
    event = BaseEvent()
    event.monitor_event('DIDAttributeChange')


@click.command()
@click.option('-f', "--file", default='/etc/oracle.conf', help='指定配置文件的路径（默认: /etc/oracle.conf）')
def organization(file):
    # TODO: 机构 运行的oracle程序"
    # 初始化配置信息
    import_plugins()
    setting.read_config2(file)
    print(setting.VAR)
    organization = OrganizationEvent()
    organization.monitor_event('DIDAttributeConfirmed')


@click.command()
@click.option('-f', "--file", default='/etc/oracle.conf', help='指定配置文件的路径（默认: /etc/oracle.conf）')
def impevent(file):
    """
    导入event数据到数据库中
    :param file: 配置文件的路径
    :return:
    """
    # global PATH
    # setting.PATH = os.path.dirname(os.path.abspath(__file__))
    import_plugins()
    # 初始化配置信息
    setting.read_config2(file)
    # business.monitor_event('DIDOwnerChanged')
    import_event()


@click.command()
@click.option('-f', "--file", default='/etc/oracle.conf', help='指定配置文件的路径（默认: /etc/oracle.conf）')
def upload_chain(file):
    """
    消费队列中的数据，进行上链操作
    :param file:
    :return:
    """
    import_plugins()
    # 初始化配置信息
    setting.read_config2(file)
    # 实例化 Contract
    provider = VAR['ETH_PROVIDER']
    contract_address = VAR['CONTRACT_ADDRESS']
    abi = VAR['CONTRACT_ABI']
    operator_private_key = VAR['OPERATOR_PRIVATEKEY']
    delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
    gas = VAR['GAS']
    gas_prise = VAR['GAS_PRISE']
    concart = Contract(provider=provider, timeout=60, contract_address=contract_address, abi=abi,
                  private_key=operator_private_key,
                  gas=int(gas),
                  gas_prise=int(gas_prise)
                  )
    # 启动 消费者程序
    cus = cusomer.CallContract(
        contracrt=concart,
        queue_name=VAR['QUEUE_NAME'],
        queue_ip=VAR['QUEUE_IP'],
        queue_port=VAR['QUEUE_PORT'],
        queue_user=VAR['QUEUE_USER'],
        queue_password=VAR['QUEUE_PASSWORD'],
        queue_vhost=VAR['QUEUE_VHOST'],
    )
    cus.cusomer()
    # from concurrent.futures import ProcessPoolExecutor
    # with ProcessPoolExecutor(max_workers=5) as executor:
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)


@click.command()
@click.option('-f', "--file", default='/etc/oracle.conf', help='指定配置文件的路径（默认: /etc/oracle.conf）')
def validation_chain(file):
    """ 用于验证数据是否上链 """
    import_plugins()
    # 初始化配置信息
    setting.read_config2(file)

    # 实例化 Contract
    provider = VAR['ETH_PROVIDER']
    contract_address = VAR['CONTRACT_ADDRESS']
    abi = VAR['CONTRACT_ABI']
    operator_private_key = VAR['OPERATOR_PRIVATEKEY']
    delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
    gas = VAR['GAS']
    gas_prise = VAR['GAS_PRISE']
    concart = Contract(provider=provider, timeout=60, contract_address=contract_address, abi=abi,
                       private_key=operator_private_key,
                       gas=gas,
                       gas_prise=gas_prise
                       )
    # 启动 消费者程序
    cus = cusomer.ValidationContract(
        contracrt=concart,
        queue_name=VAR['QUEUE_NAME_VALIDATION'],
        queue_ip=VAR['QUEUE_IP'],
        queue_port=VAR['QUEUE_PORT'],
        queue_user=VAR['QUEUE_USER'],
        queue_password=VAR['QUEUE_PASSWORD'],
        queue_vhost=VAR['QUEUE_VHOST'],
    )
    cus.cusomer()
    # from concurrent.futures import ProcessPoolExecutor
    # with ProcessPoolExecutor(max_workers=5) as executor:
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)
    #     executor.submit(cus.cusomer)

@click.command()
@click.option('-f', "--file", default='/etc/oracle.conf', help='指定配置文件的路径（默认: /etc/oracle.conf）')
def validation_chain2(file):
    import_plugins()
    # 初始化配置信息
    setting.read_config2(file)
    from oracle.cusomer import ValidationChainWorker
    jobs = []
    for i in range(3):
        p = ValidationChainWorker(i)
        jobs.append(p)
        p.start()
        print("count : {}".format(len(jobs)))
    for j in jobs:
        j.join()
    print("count22 : {}".format(len(jobs)))

if __name__ == "__main__":
    # impevent()
    army()
    # army('/Users/yuyongpeng/git/hard-chain/cport/oracle/oracle_army.conf')
    # organization('/Users/yuyongpeng/git/hard-chain/cport/oracle/oracle.conf')
    # upload_chain()
    # validation_chain2()



#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-17 12:38:25
LastEditors:  
LastEditTime: 2019-04-17 12:38:25
Description:  
"""

import json
from oracle.plugins import Processor
from oracle.database import dbs
from oracle.database import status
from oracle.dids import Did
from oracle.database.modles import Army
from oracle.setting import VAR
from oracle.logger import logger
from oracle import crypto
from pycontractsdk.contracts import Contract

@Processor.plugin_register('DIDAttributeChange', ['Solider', '士兵'])
class Solider(object):
    """
    处理 event = DIDAttributeChange 中， 身份为 士兵 的数据
    """
    def __init__(self):
        pass

    def did_attribute_change(self, contract, army_private_key, entry):
        """
        处理 event=DIDAttributeChange 的函数
        确认军人的身份之后，进行上链操作，告诉大家他是一个军人

        获得的 event 数据内容
        AttributeDict(
        {'args': AttributeDict(
          {
            'identity': '0xB2be7c8722935DB16921D64Ad0dDB5D47A52482a',
            'name': b'\xe5\xa3\xab\xe5\x85\xb5\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
            'value': b'{"@context":"https://w3id.org/did/v1","id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a#soldier",
                        "publicKey":[{"id":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a##proof",
                                      "type":"EciesVerificationKey",
                                      "controller":"did:ethr:B2be7c8722935DB16921D64Ad0dDB5D47A52482a",
                                      "publicKeyHex":"0x04521fbe1f862845cb09ca6c0cdff00417d4a3bf3c04748806eb49079f5d2d5d555c6461ecc3760bd10de1bc7c77421f02bbe3642d6d3e35ee219f84a026478297"}],
                        "authentication":[{"id":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2#proof",
                                            "type":"EciesVerificationKey",
                                            "controller":"did:ethr:DeCD6626f23bc43cB4D738cB0F2Dde922D691fD2",
                                            "publicKeyHex":"0x024af3c4d907a75e9349f362b0f854c7d74a89dad457df4f9cbf3c4f9e2f7fbcf8"}],
                        "proof":"6617fd92f9f8b9b930ff051f6c02ece8d27cf0b2912361093ce38e521bc3c4846885eec37fefdbb06ebc699fcaeb4136",
                        "created":1554184638038}
                        \n
                        04a469fb703b394646a995e6681771967beea3f1ca8fe7b370555e4c5a6dca5b071ea5d9bbec76e3b1058b54894ddff9f9d346934f74a511d17bf56e6cb892fa01
                        '
          }
        ),
        'event': 'DIDAttributeChange',
        'logIndex': 0,
        'transactionIndex': 0,
        'transactionHash': HexBytes('0xe453910d44ad566093d33593f6411fd73126564617bd1529fedce6fe42be0fa7'),
        'address': '0x5c12AED6613AaD77eC2e84e5cedd9b6Ff29A6e6A',
        'blockHash': HexBytes('0x0c602e8ad51cb5a49ab4952181a252f0bf6f55c9f3fc0cbaa043a9f2b8e5814e'),
        'blockNumber': 293710}
        )

        :param contract: pycontractsdk.contract 对象，用于上链
        :param army_private_key: 军队账号的私钥
        :param entry:
        :param entry:  获得的event数据
        :return:
        """
        # 数据存到数据库中
        event = dbs.save_army(entry)
        # 数据库中记录的是 已经上链成功 就不需要再处理了
        if event.status == status.EVENT_ETHEREUM_SUBMIT_SUCCESS:
            return
        id = event.id
        args = entry['args']
        identity = args['identity']                         # 用户的address
        name = args['name'].strip(b'\x00')                  # 身份 比如"士兵"
        value = args['value']                               # did和签名信息
        sign_text_bytes = value.split(b'\n')[-1]            # 学文的签名信息
        did_json_bytes = b''.join(value.split(b'\n')[:-1])  # did 信息 bytes
        did_json_obj = json.loads(did_json_bytes.decode())  # did json对象
        created_timestamp = did_json_obj['created']

        if not Did.verify_did(did_json_bytes.decode(), sign_text_bytes.decode()):
            logger.error('First signature verification error !')
            logger.error('did: {}'.format(did_json_bytes.decode()))
            logger.error('sign_text: {}'.format(sign_text_bytes.decode()))
            # 需要修改数据库的状态为验证签名失败
            Army.update(status=status.EVENT_SIGN_VERI_FAILED).where(Army.id == id).execute()
            return
        # 数据签名验证成功
        Army.update(status=status.EVENT_SIGN_VERI_SUCCESS).where(Army.id == id).execute()
        army_privatekey = VAR['ARMY_PRIVATEKEY']
        # ephemPublicKey 存放在did json串中 (加密者的公钥)
        ephem_public_key = did_json_obj['publicKey'][0]['publicKeyHex']
        # 得到学文加密用的iv
        iv = crypto.get_iv(str(created_timestamp))
        # 从did中得到加密串
        ciphertext = Did.get_ciphertext(did_json_obj)
        # # 解密
        original = crypto.decrypt_ecies(private_key=army_privatekey,
                                        ephemPublicKey=ephem_public_key,
                                        ciphertext=ciphertext,
                                        iv=iv).strip()
        if len(str(original)) == 0:
            Army.update(status=status.EVENT_DECRYPTION_FAILED).where(Army.id == id).execute()   # 解密失败
        else:
            Army.update(status=status.EVENT_DECRYPTION_SUCCESS).where(Army.id == id).execute()   # 解密成功

        # 解密数据 并且在 退伍士兵中查询是否有这个士兵
        (check, soldier) = crypto.check_ecies_in_database(private_key=army_private_key,
                                                          ephem_public_key=ephem_public_key,
                                                          ciphertext=ciphertext,
                                                          iv=iv)
        if check is False:
            # 数据库中不存在士兵的这条数据
            Army.update(status=status.EVENT_ETHEREUM_SOLDIER_NOTEXIST).where(Army.id == id).execute()
            logger.error('There is no data for this soldier in the database.  Verification failed ! ciphertext : {}'.fromat(ciphertext))
        if check is True:
            # 数据验证完成
            Army.update(status=status.EVENT_VERI_COMP).where(Army.id == id).execute()
            logger.info('The data of the soldier in the database. Verification success! ciphertext : {}'.format(ciphertext))

            # 生成第二个签名数据, 并且保存到数据库中
            second_sign_str = Did.sign_did(did_json_bytes.decode(), sign_text_bytes.decode(), VAR['ARMY_PRIVATEKEY'])
            Army.update(second_sign=second_sign_str).where(Army.id == id).execute()
            data = '{did_json_str}\n{first_sign}\n{second_sign}'.format(did_json_str=did_json_bytes.decode(),
                                                                        first_sign=sign_text_bytes.decode(),
                                                                        second_sign=second_sign_str)

            # 上链操作 confirmAttribute(identity, name, value)
            tf, tx_hash = contract.call_function('confirmAttribute', True, *(identity, name, data.encode()))
            # tf, tx_hash = cport.confirm_attribute(identity, name, data.encode())
            if tf is True:
                Army.update(status=status.EVENT_ETHEREUM_SUBMIT_SUCCESS, txhash=tx_hash).where(Army.id == id).execute()
            if tf is False:
                Army.update(status=status.EVENT_ETHEREUM_SUBMIT_FAILED, txhash=tx_hash).where(Army.id == id).execute()
            # Army.save()
            # cport.confirm_attribute()
            # celery异步上链操作
            # tasks.confirm_attribute.delay(cport, identity, name, value)

    def process(self, entry):
        """
        处理 士兵 的 event数据
        :param entry: 获得的event数据
        :return:
        """
        provider = VAR['ETH_PROVIDER']
        contract_address = VAR['CONTRACT_ADDRESS']
        abi = VAR['CONTRACT_ABI']
        army_private_key = VAR['ARMY_PRIVATEKEY']
        operator_private_key = VAR['OPERATOR_PRIVATEKEY']
        delegate_private_key = VAR['DELEGATE_PRIVATEKEY']
        cport = Contract(provider=provider, timeout=60, contract_address=contract_address, abi=abi,
                      private_key=operator_private_key, gas=VAR['GAS'], gas_prise=VAR['GAS_PRISE']
                      )
        self.did_attribute_change(contract=cport,
                                  army_private_key=army_private_key,
                                  entry=entry
                                  )

@Processor.plugin_register('DIDAttributeChange', ['xxx', '士兵1'])
class Soliderx(object):
    """
    处理 event = DIDAttributeChange 中， 身份为 士兵 的数据
    """
    def __init__(self):
        pass

    def process(self, entity):
        print('xxxtest')

if __name__ == '__main__':
    # 测试
    print("xx")
    processor = Processor()  # 这条数据还没有运行，就已经把插件注册好了
    processed = processor.process_identity(event='DIDAttributeChange', identity="Solider", entry="ttttt")
    processed = processor.process_identity(event='DIDAttributeChange', identity="士兵", entry="士兵")



#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-11 14:32:39
LastEditors:  
LastEditTime: 2019-04-11 14:32:39
Description:  
"""
import smtplib
from email.mime.text import MIMEText
from email.utils import formataddr
from oracle.logger import logger

def send_email(mail_subject, mail_host, mail_sender, mail_pass, mail_receiver=[], mail_body='', mail_port=465):
    """
    发送邮件的类库
    :param mail_subject:    邮件主题
    :param mail_host:       邮箱ip或域名
    :param mail_sender:     邮件发送者的邮箱
    :param mail_pass:       邮箱的密码
    :param mail_receiver:   邮件接收者的邮箱
    :param mail_body:       邮件的内容
    :param mail_port:       邮箱的端口
    :return:
    """
    ret = True
    try:
        for mail in mail_receiver:
            msg = MIMEText(mail_body, 'plain', 'utf-8')
            msg['From'] = formataddr(["sender", mail_sender])  # 括号里的对应发件人邮箱昵称、发件人邮箱账号
            msg['To'] = formataddr(["receiver", mail])  # 括号里的对应收件人邮箱昵称、收件人邮箱账号
            msg['Subject'] = mail_subject  # 邮件的主题，也可以说是标题

            server = smtplib.SMTP_SSL(mail_host, mail_port)  # 发件人邮箱中的SMTP服务器，端口是25
            server.login(mail_sender, mail_pass)  # 括号中对应的是发件人邮箱账号、邮箱密码
            server.sendmail(mail_sender, mail, msg.as_string())  # 括号中对应的是发件人邮箱账号、收件人邮箱账号、发送邮件
            server.quit()  # 关闭连接
    except Exception as e:  # 如果 try 中的语句没有执行，则会执行下面的 ret=False
        logger.error(e)
        ret = False
    return ret




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os, sys, shutil

py_version = sys.version_info[:2]

if py_version <= (2, 7):
    raise RuntimeError('On Python 2, Oracle not Support !')
elif (3, 0) < py_version < (3, 6):
    raise RuntimeError('On Python 3, Oracle requires Python 3.6 or later')


# copy oracle.conf -> /etc/oracle.conf
here = os.path.abspath(os.path.dirname(__file__))
shutil.copyfile(os.path.join(here, 'oracle.conf'), '/etc/oracle.conf')
shutil.copyfile(os.path.join(here, 'oracle_army.conf'), '/etc/oracle_army.conf')

try:
    README = open(os.path.join(here, 'README.md')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except:
    README = """
oracle 是一个智能协约的管理程序，用于导入event的数据到数据库中，和处理event数据
"""
    CHANGES = ''


setup(
    name="oracle-eth",
    version="0.0.13",
    author="yuyongpeng",
    author_email="yuyongpeng@hotmail.com",
    description=("This is a sdk of constract"),
    long_description=README + '\n\n' + CHANGES,
    # long_description_markdown_filename='README.md',
    # license="MIT",
    license=open('LICENSE').read(),
    keywords="solidity web3 constract sdk",
    url="https://gitlab.hard-chain.cn/cport/oracle",
    # packages=['oracle'],  # 需要打包的目录列表
    packages=find_packages(where='.', exclude=('tests', 'tests.*'), include=('*',)),
    # 需要安装的依赖
    install_requires=[
        # 'web3>=4.8.3',
        # 'ethereum==2.3.2',
        # 'eth-account',
        # 'eth-utils',
        'pycontractsdk',    # 对web3的封装
        'sierra-utils-rand',
        'clock',
        'peewee==3.9.3',
        'arrow==0.13.1',
        'pymysql==0.9.3',
        'celery==4.3.0',
        'pika==1.0.0',
        'kafka==1.3.5',
        'retry==0.9.2',
        'redis==3.2.1',
        'requests==2.21.0',
        #logzero
        'logbook==1.4.3',
        'toolz==0.9.0',
        'wrapt==1.11.1',
        'click==7.0',
        'random2',
        'python-common-cache==0.1',
        'wechatpy[cryptography]==1.8.1',
        'cryptography==2.6.1',
        'rsa==4.0',
    ],
    py_modules=['oracle'],
    python_requires='>=3.6',
    setup_requires=['setuptools-markdown'],
    # 添加这个选项，在windows下Python目录的scripts下生成exe文件
    # 注意：模块与函数之间是冒号:
    entry_points={'console_scripts': [
        'army = oracle.main:army',                          # 军队端 oracle
        'dpm = oracle.main:dpm',                            # 事务部端 oracle
        'organization = oracle.main:organization',          # organization 的 oracle
        'impevent = oracle.main:impevent',                  # 把 DIDRegistry.sol 中的 event 数据导入到数据库中
        'uploadchain = oracle.main:upload_chain', # 上链的消费者程序
    ]},
    # 会把这个list中的文件copy到系统的PATH路径下
    scripts=['run.sh'],
    # long_description=read('README.md'),
    classifiers=[  # 程序的所属分类列表
        "Development Status :: 3 - Alpha",
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Topic :: Utilities",
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
    ],
    zip_safe=False
)


#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
# Author: yuyongpeng@hotmail.com
# Description: 处理和did相关的数据
#

"""


from pycontractsdk import keys
import json


class Did(object):
    def __init__(self):
        pass

    @staticmethod
    def sign_did(did_json_str, first_sign_str, private_key):
        """
        对（did数据+'\n'+第一次签名的数据）进行签名
        :param did_json_str: did的json string
        :param first_sign_str: 第一次签名的数据 （学文提供，从区块链中获得）
        :param private_key: 签名用的私钥, 带有 0x 前缀
        :return: 签名后的结果 hex
        """
        sign_msg = keys.ecdsa_sign('{did_json}\n{first_sign_str}'.format(did_json=did_json_str, first_sign_str=first_sign_str),
                                   private_key_hex=private_key)
        return sign_msg

    @staticmethod
    def verify_did(did_json_str, first_sign_str):
        """
        验证第一次签名真是否正确
        :param did_json: did的json  string
        :param first_sign_str: 第一次签名的字符串 hex
        :return: 正确返回True, 错误返回False
        """
        if not isinstance(did_json_str, str):
            raise ValueError('did_json must be string')
        did_json_obj = json.loads(did_json_str)
        public_key = did_json_obj['publicKey'][0]['publicKeyHex']
        tf = keys.ecdsa_verify(did_json_str, first_sign_str, public_key)
        if tf:
            return True
        else:
            return False

    @staticmethod
    def verify_did_and_firstsign(did_json_str, first_sign_str):
        """
        验证第一次签名真是否正确
        :param did_json: did的json  string
        :param first_sign_str: 第一次签名的字符串 hex
        :return: 正确返回True, 错误返回False
        """
        if not isinstance(did_json_str, str):
            raise ValueError('did_json must be string')
        did_json_obj = json.loads(did_json_str)
        public_key = did_json_obj['publicKey'][0]['publicKeyHex']
        tf = keys.ecdsa_verify(did_json_str, first_sign_str, public_key)
        if tf:
            return True
        else:
            return False

    @staticmethod
    def get_ciphertext(did_json):
        """
        得到did中的加密串
        :param did_json:
        :return:
        """
        id = did_json['id']
        key = id.split('#')[1]
        proof = did_json['proof']
        return ''.join(['0x', proof.lstrip('0x')])
